/* histo -- plots a histogram of any data using GMT 5, and carry out
	statistical test to normal/log-normal distribtions. V1.2.   0914,1218,1019
    USAGE:
	histo file [-Ww -Xmin/max -Ymin/max -C[N][L] -S -H]
    Options:
	-Ww       -- bin width w
	-Xmin/max -- range of min & max for x-axis
	-Ymin/max -- range of min & max for y-axis
	    (assigned ranges alter only graph appearance not analyses)
	-C[N][L]  -- curves of normal (default/N) or/and log-normal (L) dists
	-S        -- carry out Chi-2 and K-S tests to normal or/and log-normal
		     distributions (which dist depends on the C-option).
	-H        -- help message
    Input file:
	# comment lines with # (or %) at the beginning at any places
	# one datum per line at 1st column
	    x1 [y1, z1, .. -- any extra data are ignored]
	    x2
	    .
	    .
    Output file:
	PS graph of histogram is stored in file.ps (extension is removed)
	When s option, results of statistical test are written to stdout
    Variables:
	N,X -- number & array of data (X[0]--X[N-1])
	Xmin,Xmax -- range of data X to be plotted
	    (Do not confuse with minx & maxx which are real ones)
	Ymin,Ymax -- range of frequency Y to be plotted (Ymin>=0)
	dX -- bin width for X
	Nfrq -- bin number (default NBINDFLT=22 including two edge zero bins)
	    (Do not confuse Nfrq with nbin for Chi2 test)
	Frq -- frequency array for each bin (in default, Frq[0]=Frq[Nfrq-1]=0)
	    (Do not confuse Frq[] with bin[] & ebin[] for Chi2 test)
	NFRC -- number of steps to draw distribution curves (now 200)
*/

#include <stdlib.h>
#include "histo.h"

#define HEAD "Histogram and distribution test of %s.\n\
  PS file: %s\n\
  N     Mean       Stdev      Max        Min\n\
  %-4d %#- 10.4g %#- 10.4g %#- 10.4g %#- 10.4g\n"

char File[NF], Filet[NF], Filep[NF];
unsigned int Opt=0;
int N, Nfrq;
double *X, Xmax, Xmin, dX, Ymax, Ymin=0., *Frq, Mxl, Sxl, Mxg, Sxg, Frcn[NFRC], Frcl[NFRC];

int main(argc, argv)
int argc;
char *argv[];
{
	double mx, sx, maxx, minx;

	hst_cmd(argc, argv);
	hst_dat(&mx, &sx, &maxx, &minx);
	fprintf(stdout, HEAD, File, Filep, N, mx, sx, maxx, minx);
	hst_hist(mx, sx);
	if (Opt & STAT_BIT) hst_stat(mx, sx);
	hst_mfree();
/*	sys_call(3, "evince ", Filep, " &"); */
	sys_call(2, "rm -f ", Filet);
}
#undef HEAD
