/* histo.h */

#include "std.h"

#define USAGE	"Usage: histo file [-Ww -Xmin/max -Ymin/max -C[N][L] -S -H]"
#define HELP	"HELP MESSAGE OF HISTO\n\
  Histogram of scalar data V1.21, H. Tanaka, 2021,2018.\n\
  Optional approximation by normal or log-normal distribution curves\n\
  and statistical test of Chi-2 and Kolmogorov-Smirnov tests.\n\
\n\
Usage:\n\
  histo file [-Ww -Xmin/max -Ymin/max -C[N][L] -S -H]\n\
\n\
Options:\n\
  -Ww       -- bin width w\n\
  -Xmin/max -- range of min and max for x-axis\n\
  -Ymin/max -- range of min and max for y-axis\n\
              (options X/Y alter only the graph appearance)\n\
  -C[N][L]  -- curves of normal(N)/log-normal(L) distributions\n\
  -S        -- Chi-2 and K-S tests to normal/log-normal\n\
               distributions (depends on params in C-option).\n\
  -H        -- help message\n\
\n\
Input file:\n\
  # comment line with # (or %) at the beginning at any places\n\
  # one datum per line at 1st column\n\
  x1 [y1, z1, .. -- any extra data are ignored]\n\
  x2\n\
   .\n\
   .\n\
\n\
Output file:\n\
  PS graph of histogram is created with the name \"file.ps\".\n\
  When S option, results of statistical test are written to stdout.\n\
(Press Q to Quit)"

#define NF	200
#define NBUF	250
#define NFRC	200

#define BIN_WIDTH_BIT	1
#define XRANGE_BIT	2
#define YRANGE_BIT	4
#define NORM_BIT	8
#define LOGNORM_BIT	16
#define STAT_BIT	32

#define NBINDFLT	22
#define RANGEFACT	0.1

#define MAPWIDTH	12.5
#define MAPHEIGHT	10.0
#define GSET	" PS_MEDIA A4 PROJ_LENGTH_UNIT c MAP_FRAME_TYPE plain MAP_FRAME_PEN thick,black \
MAP_TICK_PEN_PRIMARY thin,black MAP_GRID_PEN_PRIMARY thinner,black \
FONT_TITLE 18p,Helvetica-Bold,black FONT_LABEL 14p,Helvetica-Bold,black \
FONT_ANNOT_PRIMARY 12p,Helvetica,black MAP_POLAR_CAP none FORMAT_FLOAT_MAP %.4g \
FORMAT_TIME_STAMP \"%Y %b %d\""

/* functions */

#define MAX(a,b) ((a >= b) ? a : b)

void hst_cmd();
void hst_dat(), hst_mfree();
void hst_hist(), plot_hist(), plot_histK(), plot_curven(), plot_curvenK(), \
	plot_curvel(), plot_curveld(), create_curve(), g_xmax_dx();
void hst_stat(), chi2test_norm(), chi2test_lognorm(), kstest_norm(), kstest_lognorm();
