/* hst_cmd -- command interpretation for histo */

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "histo.h"

#define EPS	1.e-20
#define HUGE	1.e+20

extern char File[NF], Filet[NF], Filep[NF];
extern unsigned int Opt;
extern double dX, Xmax, Xmin, Ymax, Ymin;

void hst_cmd(argc, argv)
int argc;
char *argv[];
{
	char *s, s1[20], s2[20], buf[NBUF], *ptr;
	int flagf=0, i=0;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
	if (*argv[argc] == '-'){
		s = argv[argc] + 1;
		switch (toupper(*s)){
		case 'W':
			getoption(s+1, 1, s1);
			if (s1[0] == '\0') error("Assign bin width!");
			dX = atof(s1);
			if (dX<EPS || dX>HUGE) error("Too small or large bin width!");
			Opt |= BIN_WIDTH_BIT;
			break;
		case 'X':
			getoption(s+1, 2, s1, s2);
			if (s1[0]=='\0' || s2[0]=='\0') error("Assign min & max of X-range!");
			Xmin = atof(s1);   Xmax = atof(s2);
			if (Xmin<-HUGE || Xmax>HUGE || Xmin>=Xmax)
				error("Error in the range of X-axis!");
			Opt |= XRANGE_BIT;
			break;
		case 'Y':
			getoption(s+1, 2, s1, s2);
			if (s1[0]=='\0' || s2[0]=='\0') error("Assign min & max of Y-range!");
			Ymin = atof(s1);   Ymax = atof(s2);
			if (Ymin<0. || Ymax>HUGE || Ymin>=Ymax)
				error("Error in the range of Y-axis!");
			Opt |= YRANGE_BIT;
			break;
		case 'C':
			if (*(++s) == '\0'){
				Opt |= NORM_BIT; break;
			}
			else if (toupper(*s) == 'N'){
				Opt |= NORM_BIT;
				if (*(s+1) == '\0') break;
				else if (toupper(*(s+1)) == 'L'){
					Opt |= LOGNORM_BIT; break;
				}
				else error("Unknown parameter in C-option!");
			}
			else if (toupper(*s) == 'L'){
				Opt |= LOGNORM_BIT;
				if (*(s+1) == '\0') break;
				else if (toupper(*(s+1)) == 'N'){
					Opt |= NORM_BIT; break;
				}
				else error("Unknown parameter in C-option!");
			}
			else error("Unknown parameter in C-option!");
			break;
		case 'S':
			Opt |= STAT_BIT;
			break;
		case 'H':
			showless(HELP);
			break;
		default:
			error("Unknown option!");
			break;
		}
	}
	else {
		strcpy(File, argv[argc]);
		flagf++;
	}
	}
	if (flagf == 0)
		error("Assign a data file!");
	else if (flagf > 1)
		error("Assign only one file!");
	strcpy(buf, File);
	if ((ptr=strtok(buf,".")) == NULL) error("Error of data file at strtok!");
	strcpy(Filet, ptr);   strcat(Filet, ".TMP");
	strcpy(Filep, ptr);   strcat(Filep, ".ps");
}
#undef EPS
#undef HUGE
