/* histo2.h -- definitions and functions for histo2   1218 */

#include "std.h"

#define USAGE	"Usage: histo2 file1 file2 [-Ww -Xmin/max -Ymin/max -C[N][L] -T[E] -S -H]"
#define HELP	"HELP MESSAGE OF HISTO2\n\
  Histograms of two datasets V1.01, H.Tanaka, 2021,2018.\n\
  Optionally tests whether the two data share the same distribution,\n\
  using Chi-2 and Kolmogorov-Smirnov tests. Also draws curves of\n\
  normal/log-normal distributions using the means & std deviations.\n\
\n\
Usage:\n\
  histo2 file1 file2 [-Ww -Xmin/max -Ymin/max -C[N][L] -T -S -H]\n\
\n\
Options:\n\
  -Ww       -- bin width w\n\
  -Xmin/max -- range of min and max for x-axis\n\
  -Ymin/max -- range of min and max for y-axis\n\
              (options X/Y alter only the graph appearance)\n\
  -C[N][L]  -- curves of normal(N)/log-normal(L) distributions\n\
  -T[E]     -- plot a histogram of the totaled data with y-axis\n\
               down-scaled (default)/the same (E) as each data\n\
  -S        -- Chi-2 and K-S tests to common distribution and\n\
               normal/log-normal distributions (with C-option).\n\
  -H        -- help message\n\
\n\
Input file:\n\
  Two separate data files should be supplied with the format as,\n\
  # comment line with # (or %) at the beginning at any places\n\
  # one datum per line at 1st column\n\
  x1 [y1, z1, .. -- any extra data are ignored]\n\
  x2\n\
   .\n\
   .\n\
\n\
Output file:\n\
  PS figure is created with the name \"file1-file2.ps\".\n\
  When S option, results of statistical test are written to stdout.\n\
(Press Q to Quit)"

#define NF	200
#define NBUF	250
#define NFRC	200

#define BIN_WIDTH_BIT	1
#define XRANGE_BIT	2
#define YRANGE_BIT	4
#define NORM_BIT	8
#define LOGNORM_BIT	16
#define TOTALHST_BIT	32
#define TLSCALE_BIT	64
#define STAT_BIT	128

#define NBINDFLT	22
#define RANGEFACT	0.1

#define MAPWIDTH	12.5
#define MAPHEIGHT	10.0
#define MAPHEIGHT2	6.0
#define GSET	" PS_MEDIA A4 PROJ_LENGTH_UNIT c MAP_FRAME_TYPE plain MAP_FRAME_PEN thick,black \
MAP_TICK_PEN_PRIMARY thin,black MAP_GRID_PEN_PRIMARY thinner,black \
FONT_TITLE 18p,Helvetica-Bold,black FONT_LABEL 14p,Helvetica-Bold,black \
FONT_ANNOT_PRIMARY 12p,Helvetica,black MAP_POLAR_CAP none FORMAT_FLOAT_MAP %.4g \
FORMAT_TIME_STAMP \"%Y %b %d\""

/* functions */

#define MAX(a,b) ((a >= b) ? a : b)
#define MIN(a,b) ((a < b) ? a : b)

void hst2_cmd();
void hst2_dat(), hst2_read_x(), cal_maxy(), cal_maxyt(), max_norm_crv(), max_lnorm_crv(), \
	hst2_mfree();
void hst2_hist(), cal_curven_data(), cal_curvel_data(), wrt_xy_data(), wrt_a_text(), \
	g_xmax_dx();
void hst2_stat(), chi2test_two(), kstest_two(), chi2test_norm(), chi2test_lognorm(), \
	kstest_norm(), kstest_lognorm();
