/* hst2_cmd -- command interpretation for histo2 */

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "histo2.h"

#define EPS	1.e-20
#define HUGE	1.e+20

extern char File1[NF], File2[NF], Filet[NF], Filep[NF];
extern unsigned int Opt;
extern int N1, N2, Nt;
extern double dX, Xmax, Xmin, Ymax, Ymin;

void hst2_cmd(argc, argv)
int argc;
char *argv[];
{
	FILE *fp1, *fp2;
	char *s, s1[30], s2[30], buf[NBUF], *ptr;
	int flagf=0, i, n;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
	if (*argv[argc] == '-'){
		s = argv[argc] + 1;
		switch (toupper(*s)){
		case 'W':
			getoption(s+1, 1, s1);
			if (s1[0] == '\0') error("Assign bin width!");
			dX = atof(s1);
			if (dX<EPS || dX>HUGE) error("Too small or large bin width!");
			Opt |= BIN_WIDTH_BIT;
			break;
		case 'X':
			getoption(s+1, 2, s1, s2);
			if (s1[0]=='\0' || s2[0]=='\0') error("Assign min & max of X-range!");
			Xmin = atof(s1);   Xmax = atof(s2);
			if (Xmin<-HUGE || Xmax>HUGE || Xmin>=Xmax)
				error("Error in the range of X-axis!");
			Opt |= XRANGE_BIT;
			break;
		case 'Y':
			getoption(s+1, 2, s1, s2);
			if (s1[0]=='\0' || s2[0]=='\0') error("Assign min & max of Y-range!");
			Ymin = atof(s1);   Ymax = atof(s2);
			if (Ymin<0. || Ymax>HUGE || Ymin>=Ymax)
				error("Error in the range of Y-axis!");
			Opt |= YRANGE_BIT;
			break;
		case 'C':
			if (*(++s) == '\0'){
				Opt |= NORM_BIT; break;
			}
			else if (toupper(*s) == 'N'){
				Opt |= NORM_BIT;
				if (*(s+1) == '\0') break;
				else if (toupper(*(s+1)) == 'L'){
					Opt |= LOGNORM_BIT; break;
				}
				else error("Unknown parameter in C-option!");
			}
			else if (toupper(*s) == 'L'){
				Opt |= LOGNORM_BIT;
				if (*(s+1) == '\0') break;
				else if (toupper(*(s+1)) == 'N'){
					Opt |= NORM_BIT; break;
				}
				else error("Unknown parameter in C-option!");
			}
			else error("Unknown parameter in C-option!");
			break;
		case 'T':
			Opt |= TOTALHST_BIT;
			if (*(++s) == '\0') break;
			else if (toupper(*s) == 'E'){
				Opt |= TLSCALE_BIT;
			}
			else error("Unknown parameter in T-option!");
			break;
		case 'S':
			Opt |= STAT_BIT;
			break;
		case 'H':
			showless(HELP);
			break;
		default:
			error("Unknown option!");
			break;
		}
	}
	else {
		if (flagf == 0) strcpy(File2, argv[argc]);
		else if (flagf == 1) strcpy(File1, argv[argc]);
		flagf++;
	}
	}
	if (flagf < 2) error("Assign two data files!");
	else if (flagf > 2) error("Assign only two files!");
	strcpy(buf, File1);
	if ((ptr=strtok(buf,".")) == NULL) error("Error of data file!");
	strcpy(Filet, ptr);   strcat(Filet, "-");
	strcpy(Filep, ptr);   strcat(Filep, "-");
	strcpy(buf, File2);
	if ((ptr=strtok(buf,".")) == NULL) error("Error of data file!");
	strcat(Filet, ptr);   strcat(Filet, ".TMP");
	strcat(Filep, ptr);   strcat(Filep, ".ps");
	if ((fp1=fopen(File1,"r")) == NULL) error("Error of file open!");
	if ((fp2=fopen(File2,"r")) == NULL) error("Error of file open!");
	i = 0;
	while ((n=get_s(fp1,1,s1)) != EOF){
		if (n==0 || s1[0]=='#' || s1[0]=='%') continue;
		i++;
	}
	N1 = i;
	i = 0;
	while ((n=get_s(fp2,1,s1)) != EOF){
		if (n==0 || s1[0]=='#' || s1[0]=='%') continue;
		i++;
	}
	N2 = i;
	Nt = N1 + N2;
	fclose(fp1);   fclose(fp2);
}
#undef EPS
#undef HUGE
