/* pdemag -- vector or great circle analyses of the progressive demagnetization data
		V.1.3.1 -- minor errors are amended. Compatibility with GMT 5.4.3,
			gcc 7.4.0, and Ubuntu 18.04 is ascertained.   052619
		V.1.3 -- modified for use with GMT5.2.1 & evince. Problematic wrt_tmp() was
			simplified to be used with sprintf().   031918
		V.1.2 -- print out of MDF & DF for 1/4 is introduced   052506
		V.1 -- modified from pinte   091003
	Usage, Options, Input & Output files:
		Please see HELP given at pdmg_cmd().
	Basic idea of the program:
		pdemag read only one file which includes data of multiple samples.
		name, azim, plun, strik, dip of (i+1)th sample are stored in Sampl[i],
		Azim[i], Plun[i], Strik[i], Dip[i].  All other data are stored in series
		in Stepd[j], Magd[j], Incd[j], ...  Which sample starts at which line is
		stored in Idsmpl[i].  pdmg_smpldat(i) takes data of (i+1)th sample from
		Stepd[j], Magd[j], ... and stores them to Xs[l][k], Ys[l][k], ... where
		l denotes the orientation of core, in situ, and tilted for 0, 1, and 2.
		pdmg_gmtdat(i) sets up necessary GMT plot data, Ieq[l][k], Deq[l][k],...
		for the present (i+1)th sample for each of orientation l.  Any plotting
		or calculation are made for the present sample only but for all orientations.
		Hence, pdemag keeps the processed data for only the present sample being analyzed.
	External variables for measurement data:
		Filed -- data file name
		Filew -- name of a file which PCA/GTC results are written to
		Frame -- data coordinate, where 0, 1, & 2 are core, in situ, & tilted, respectively
		Nd -- total number of data or samples
		Nstep -- number of steps for the present sample
		Sampl[i] -- name of (i+1)th sample
		Idsmpl[i] -- points beginning of (i+1)th sample on j of Magd[j], ...
		Azim[i], Plun[i], Strike[i], Dip[i]  -- azimuth, plunge, ... of (i+1)th sample
		Stepd[j] -- content of step in (j+1)th data line
		Magd[j], Incd[j], Decd[j] -- mag, inc, dec of (j+1)th data line
		Chid[j] -- chi of (j+1)th data line
		Step[k] -- kth step (0th is NRM data)
		Xs[l][k], Ys[l][k], Zs[l][k] -- (x, y, z) for kth step with orientation l
		Mag[k], Is[l][k], Ds[l][k] -- (m, i, d) for kth step with orientation l
		Chi[k] -- chi for kth step
	External variables for GMT plot data:
		Fpt -- file pointer to a TMP file used in GMT commands
		Fpw -- file pointer to a PCA/GTC results file
		Ida[k] -- array of k for kth step used for annotation
		Ieq[l][k], Deq[l][k], Ieqr[l][k] -- (inc,dec) & flipped inc for equal area
		Xzj1[l], Xzj2[l], Yzj1[l], Yzj2[l] -- define the area plotted for Zijderveld
		Yzjyr1[l], Yzjyr2[l], Yzjzr1[l], Yzjzr2[l] -- use for draw a PCA line on Zj plot
		Gapzj[l], dXzj[l] -- gap length & tick mark point for Zijderveld
		Xzj[l][k], Yzj[l][k], Zzj[l][k] -- (x,y,z) on Zijderveld with orientation l
		Yjn[k] -- Jn on Jn-step curve
		Yjnmax -- maximum range of Jn-step curve
		dYjna, dYjnt -- annotation & tick intervals for Jn-step curve
		Ych[k] -- chi on chi-step curve
		Ychmin, Ychmax -- lower & upper limits of chi-axis
		dYcha, dYcht -- annotation & tick intervals for chi-step curve
		Factzj[l] -- factor in ZJ plot, giving inter part if multiplied to sample data
		Scalzj -- overall range factor in the Zijderveld diagram.
			To enlarge make Scalzj small, ie., view smaller region.
		Ux[l], Uy[l], Uz[l] -- eigen vector of PCA for orientation frame l
			PCA analysis is made once for the present frame, and (Ux,Uy,Uz) for
			other frames are transformed by Azim, Plun, Strik, Dip.
		Iu[l], Du[l] -- Inc & Dec of the eigen vector for frame l
		Madc, Mad -- critical & dtermined MAD of PCA for Line or GTC analysis
		Theta -- difference angle between the u and the anchored direction(?)
		Ip[l], Dp[l] -- Inc & Dec of the GTC pole for coordinate frame l
		Ia1[l], Da1[l], Ia2[l], Da2[l] -- Inc & Dec of constraint points A1 & A2 on
						the GTC plane for coordinate frame l
		I1, I2 -- selected range of steps for Line & GTC with Mad<Madc
*/

#define COMMAND "+,-,0,9,C,S,B,L,G,M,R,A,T,D,W,P,N,O,E,Z,H,Q?"
#define HELPCMD "Commands:\n\
  \e[1m+\e[m-next, \e[1m-\e[m-back, \e[1m0\e[m-first, \e[1m9\e[m-last, \e[1mC\e[more, in-\e[1mS\e[mitu, \e[1mB\e[med-corrected,\n\
  \e[1mL\e[mine-pca, \e[1mG\e[mtc-pca, critical-\e[1mM\e[mad, o\e[1mR\e[migin-on/off, \e[1mA\e[mnchor-on/off, au\e[1mT\e[mo-on/off, \n\
  \e[1mD\e[melete-results, \e[1mW\e[mrite-results, \e[1mP\e[msfile, e\e[1mN\e[mlarge-scale, ann\e[1mO\e[mtate-on/on2/off,\n\
  \e[1mE\e[mqual-area, \e[1mZ\e[mijderveld, \e[1mH\e[melp, \e[1mQ\e[muit"

#include <ctype.h>
#include "pdemag.h"
#include "pdmg_gmtvars.h"

FILE *Fpt, *Fpw;
char **Sampl, **Stepd, Filed[WFILE], Filew[WFILE], Sdum[LSIZ_COM];
unsigned int Opt = 0, Frame  = SITU;
int Nd, Nstep, I1, I2, *Idsmpl, *Ida;
double *Azim, *Plun, *Strik, *Dip, *Magd, *Incd, *Decd, *Chid, *Step, Stepmax, *Mag, *Chi, \
	**Is, **Ds, **Xs, **Ys, **Zs;
double **Ieq, **Deq, **Ieqr, Xzj1[3], Xzj2[3], Yzj1[3], Yzj2[3], Yzjyr1[3], Yzjyr2[3], Yzjzr1[3], \
	Yzjzr2[3], Gapzj[3], dXzj[3], **Xzj, **Yzj, **Zzj, *Yjn, Yjnmax, dYjna, dYjnt, *Ych, \
	Ychmax, Ychmin, dYcha, dYcht, Xjnmax, dXjna, dXjnt, Factzj[3], Scalzj = 1., \
	Ux[3], Uy[3], Uz[3], Iu[3], Du[3], Madc, Mad, Theta, Ip[3], Dp[3], \
	Ia1[3], Da1[3], Ia2[3], Da2[3];

int main(argc, argv)
int argc;
char *argv[];
{
	FILE *fpr;
	int i = 0, c, nline;

	Opt |= ZJ_TOP_BIT;
	Madc = 5.;
	if ((fpr = pdmg_cmd(argc, argv, Filed)) == NULL) error("Error in fopen!");
	nline = pdmg_dat(fpr);
	fclose(fpr);
	fprintf(stdout, "Progressive demagnetization from %s\n", Filed);
	Nstep = pdmg_smpldat(i, &I1, &I2);
	fprintf(stdout, "%s\n", Sampl[i]);
	pdmg_gmtdat(i);
	pdmg_plot(i);
	sys_call(3, "evince ", FILEP, " &");
/*	sys_call(3, "gv -nodsc -watch ", FILEP, " &"); */
/*	sys_call(3, "gv -nodsc -watch -scale=0.7 ", FILEP, " &"); */
/*	sys_call(3, "gv --nodsc --watch ", FILEP, " &"); */
	while ((c = pdmg_comd(COMMAND)) != ZERO){
		switch(toupper(c)){
		case '+':
			go_next(&i);  break;
		case '-':
			go_back(&i);  break;
		case '0':
			go_first(&i);  break;
		case '9':
			go_last(&i);  break;
		case 'C':
			core_orient(i);  break;
		case 'S':
			situ_orient(i);  break;
		case 'B':
			bed_orient(i);  break;
		case 'L':
			do_pcaline(i);  break;
		case 'G':
			do_pcagtc(i);  break;
		case 'M':
			critical_mad(i);  break;
		case 'R':
			set_pcaorigin(i);  break;
		case 'A':
			anchor_origin(i);  break;
		case 'T':
			set_pcaauto(i);  break;
		case 'D':
			delete_results(i);  break;
		case 'W':
			write_results(i);  break;
		case 'P':
			out_psfile(i);  break;
		case 'N':
			enlarge_scale(i);  break;
		case 'O':
			put_annot(i);  break;
		case 'E':
			change_eq(i);  break;
		case 'Z':
			change_zj(i);  break;
		case 'H':
			fprintf(stdout, "%s\n", HELPCMD);  break;
		default:
			break;
		}
	}
/*	free_all(nline); -- DOES NOT WORK? */
	quit_gv();
	sys_call(2, "rm -f ", FILET);
}

/* quit_gv -- force to kill the running Ghost View */

void quit_gv()
{
	FILE *fp;
	char *file1 = "PDEMAG.TM2", *file2 = "PDEMAG.TM3", s1[20];

	sys_call(2, "ps a >", file1);
	sys_call(4, "grep \"evince\" ", file1, " >", file2);
/*	sys_call(4, "grep \"gv -nodsc -watch -scale=0.7\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv -nodsc -watch\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv --nodsc --watch\" ", file1, " >", file2); */
	if ((fp = fopen(file2, "r")) == NULL) error("Error at quit_gv!");
	get_s(fp, 1, s1);
	fclose(fp);
	sys_call(2, "kill ", s1);
	sys_call(2, "rm -f ", file1);   sys_call(2, "rm -f ", file2);
}
#undef COMMAND
#undef HELPCMD
