#include <math.h>
#include "pdemag.h"

#define MAX(a,b) ((a > b) ? a : b)
#define MIN(a,b) ((a < b) ? a : b)
#define EPS 1.e-20

extern unsigned int Opt, Frame;
extern int Nstep, *Ida;
extern double Stepmax, *Mag, *Chi, **Is, **Ds, **Xs, **Ys, **Zs;
extern double **Ieq, **Deq, **Ieqr, Xzj1[3], Xzj2[3], Yzj1[3], Yzj2[3], Yzjyr1[3], Yzjyr2[3], \
	Yzjzr1[3], Yzjzr2[3], Gapzj[3], dXzj[3], **Xzj, **Yzj, **Zzj, *Yjn, Yjnmax, dYjna, dYjnt, \
	*Ych, Ychmax, Ychmin, dYcha, dYcht, Xjnmax, dXjna, dXjnt, Factzj[3], Scalzj;

/* pdmg_gmtdat -- set up all the necessary GMT plot data for nth sample */

void pdmg_gmtdat(n)
int n;
{
	int i;

	for (i = 0; i < Nstep; i++) Ida[i] = i;
	set_eqdata();
	for (i = 0; i < 3; i++)
		Factzj[i] = set_zjdata(Nstep, Xs[i], Ys[i], Zs[i], Xzj1+i, Xzj2+i, Yzj1+i, Yzj2+i, \
			Gapzj+i, dXzj+i, Xzj[i], Yzj[i], Zzj[i]);
	set_jndat();
	if (set_chidat() > 0)
		Opt |= PLOT_CHI_BIT;
	else 
		Opt &= ~PLOT_CHI_BIT;
	Opt |= AUTO_BIT;  Opt |= ORIGIN_BIT;  Opt &= ~ANCHOR_BIT;
	Opt &= ~LINE_BIT;  Opt &= ~GTC_BIT;  Opt &= ~DRAW_BIT;
	Scalzj = 1.;
}

/* set_eqdata */

void set_eqdata()
{
	int i, j;

	for (i = 0; i < 3; i++)
		for (j = 0; j < Nstep; j++){
			Ieq[i][j] = Is[i][j];   Deq[i][j] = 360. - Ds[i][j];
			Ieqr[i][j] = -Is[i][j];
		}
}
 
/* set_zjdata */

double set_zjdata(n, x, y, z, x1, x2, y1, y2, gap, dx, xz, yz, zz)
int n;
double *x, *y, *z, *x1, *x2, *y1, *y2, *gap, *dx, *xz, *yz, *zz;
{
	int i;
	double xmin, xmax, ymin, ymax, zmin, zmax, Xmin, Xmax, Ymin, Ymax, \
		Xcnt, Ycnt, width, height, length, order, factr, facte = 1.2, axmax, \
		d1, d2;

	maxmin(n, x, &xmax, &xmin);
	maxmin(n, y, &ymax, &ymin);
	maxmin(n, z, &zmax, &zmin);
	Xmax = xmax;
	if (Xmax < 0.) Xmax = 0.;
	Xmin = xmin;
	if (Xmin > 0.) Xmin = 0.;
	Ymax = MAX(ymax, zmax);
	if (Ymax < 0.) Ymax = 0.;
	Ymin = MIN(ymin, zmin);
	if (Ymin > 0.) Ymin = 0.;
	width = Xmax - Xmin;   Xcnt = (Xmax + Xmin)/2.;
	height = Ymax - Ymin;   Ycnt = (Ymax + Ymin)/2.;
	if (width >= height){
		length = width;
		order = floor(log10(length));
		factr = pow(10., -order);
		*x1 = (Xcnt - (Xcnt - Xmin)*facte)*factr;
		*x2 = (Xcnt + (Xmax - Xcnt)*facte)*factr;
		*y2 = (Ycnt + 0.5*length*facte)*factr;
		*y1 = (Ycnt - 0.5*length*facte)*factr;
	}
	else {
		length = height;
		order = floor(log10(length));
		factr = pow(10., -order);
		*x1 = (Xcnt - 0.5*length*facte)*factr;
		*x2 = (Xcnt + 0.5*length*facte)*factr;
		*y2 = (Ycnt + (Ymax - Ycnt)*facte)*factr;
		*y1 = (Ycnt - (Ycnt - Ymin)*facte)*factr;
	}
	d1 = MAX(fabs(*x1), fabs(*x2));   d2 = MAX(fabs(*y1), fabs(*y2));
	axmax = MAX(d1, d2);
	*dx = set_dX(axmax);
	*gap = 0.01*length*factr;
	for (i = 0; i < n; i++){
		xz[i] = x[i]*factr;   yz[i] = y[i]*factr;   zz[i] = z[i]*factr;
	}
	return(factr);
}

/* set_dX -- used in set_zjdata() for Zijderveld diagram */

double set_dX(axmax)
double axmax;
{
        if (axmax < 0.8) return(0.5);
        if (axmax < 1.8) return(1.);
        if (axmax < 3.8) return(2.);
        else return(5.);
}

/* set_jndat -- set Jn-step plot */

void set_jndat()
{
	int i;
	double ymax, ymin;

	if (Mag[0] < EPS) error("No value for room-T Mag!");
	for (i = 0; i < Nstep; i++)
		Yjn[i] = Mag[i]/Mag[0];
	maxmin(Nstep, Yjn, &ymax, &ymin);
	if (ymax < 1.2){
		Yjnmax = 1.2;   dYjna = 0.5;   dYjnt = 0.1;
	}
	else if (ymax < 1.4){
		Yjnmax = 1.4;   dYjna = 0.5;   dYjnt = 0.1;
	}
	else if (ymax < 1.5){
		Yjnmax = 1.5;   dYjna = 0.5;   dYjnt = 0.1;
	}
	else if (ymax < 1.6){
		Yjnmax = 1.6;   dYjna = 0.5;   dYjnt = 0.1;
	}
	else if (ymax < 2.0){
		Yjnmax = 2.0;   dYjna = 0.5;   dYjnt = 0.5;
	}
	else if (ymax < 3.0){
		Yjnmax = 3.0;   dYjna = 1.0;   dYjnt = 1.0;
	}
	else {
		Yjnmax = 5.0;   dYjna = 1.0;   dYjnt = 1.0;
	}
	if (Stepmax < 110.){
		Xjnmax = 110.;   dXjna = 50.;   dXjnt = 10.;
	}
	else if (Stepmax < 210.){
		Xjnmax = 210.;   dXjna = 100.;   dXjnt = 20.;
	}
	else if (Stepmax < 520.){
		Xjnmax = 520.;   dXjna = 100.;   dXjnt = 50.;
	}
	else if (Stepmax < 660.){
		Xjnmax = 660.;   dXjna = 100.;   dXjnt = 100.;
	}
	else {
		Xjnmax = 750.;   dXjna = 500.;   dXjnt = 100.;
	}
}

/* set_chidat -- set chi-step plot.
		returns -1 when no chi data
*/

int set_chidat()
{
	int i;
	double ymax, ymin;

	if (Chi[0] < EPS) return(-1);
	for (i = 0; i < Nstep; i++)
		Ych[i] = Chi[i]/Chi[0];
	maxmin(Nstep, Ych, &ymax, &ymin);
	if (ymax < 1.2 && ymin > 0.8){
		Ychmax = 1.2;   Ychmin = 0.8;   dYcha = 0.2;   dYcht = 0.1;
	}
	else if (ymax < 1.4 && ymin > 0.6){
		Ychmax = 1.4;   Ychmin = 0.6;   dYcha = 0.4;   dYcht = 0.1;
	}
	else if (ymax < 1.5 && ymin > 0.5){
		Ychmax = 1.5;   Ychmin = 0.5;   dYcha = 0.5;   dYcht = 0.1;
	}
	else if (ymax < 1.6 && ymin > 0.4){
		Ychmax = 1.6;   Ychmin = 0.4;   dYcha = 0.6;   dYcht = 0.2;
	}
	else if (ymax < 2.0){
		Ychmax = 2.0;   Ychmin = 0.0;   dYcha = 0.5;   dYcht = 0.5;
	}
	else if (ymax < 3.0){
		Ychmax = 3.0;   Ychmin = 0.0;   dYcha = 1.0;   dYcht = 1.0;
	}
	else {
		Ychmax = 5.0;   Ychmin = 0.0;   dYcha = 1.0;   dYcht = 1.0;
	}
	return(1);
}
#undef MAX
#undef MIN
#undef EPS
