#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pdemag.h"

extern char **Sampl, Filed[WFILE];
extern unsigned int Opt;
extern int Frame, Nstep, I1, I2;
extern double *Azim, *Plun, *Strik, *Dip, **Is, **Ds, Ux[3], Uy[3], Uz[3], Ip[3], Dp[3], \
		Madc, Mad, Ia1[3], Da1[3], Ia2[3], Da2[3];

static int nr;
static double dumm, dumx0, dumy0, dumz0, dumx1, dumy1, dumz1;

/* pdmg_gtc -- Principal component analysis of a great circle on equal area plot */

void pdmg_gtc(n)
int n;
{
	nr = I2 - I1 + 1;
	pcagtc(Is[Frame]+I1, Ds[Frame]+I1, nr, Ip+Frame, Dp+Frame, &Mad);
	pdmg_gtc_parm(Frame);
	magcar(1., Ip[Frame], Dp[Frame], Ux+Frame, Uy+Frame, Uz+Frame);
	if (Frame == SITU) situ_ctg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame == CORE) core_stg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame == TILT) tilt_csg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
}

/* pdmg_gtca -- Automatic PCA simplified from Kirshvink (1980) */

int pdmg_gtca(n)
int n;
{
	int c, i1, i2;

	nr = I2 - I1 + 1;
	if (nr < 3) error("nr=2 at pdmg_gtca!\n");
	else {
		c = pcagtca(Is[Frame]+I1,Ds[Frame]+I1,nr,Madc,Ip+Frame,Dp+Frame,&i1,&i2,&Mad);
		if (c == ZERO) return ZERO;
		I1 = i1;  I2 = i2;
		pdmg_gtc_parm(Frame);
		magcar(1., Ip[Frame], Dp[Frame], Ux+Frame, Uy+Frame, Uz+Frame);
		if (Frame == SITU) situ_ctg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
		else if (Frame == CORE) core_stg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
		else if (Frame == TILT) tilt_csg(n, Ux[Frame], Uy[Frame], Uz[Frame]);
		return +1;
	}
}

/* situ_ctg -- transfer in situ GTC to core & tilted frames */

void situ_ctg(n, ux, uy, uz)
int n;
double ux, uy, uz;
{
	coren(Azim[n], Plun[n], ux, uy, uz, Ux, Uy, Uz);
	carmag(Ux[0], Uy[0], Uz[0], &dumm, Ip, Dp);
	bedcor(Strik[n], Dip[n], ux, uy, uz, Ux+2, Uy+2, Uz+2);
	carmag(Ux[2], Uy[2], Uz[2], &dumm, Ip+2, Dp+2);
	magcar(1., Ia1[1], Da1[1], &dumx0, &dumy0, &dumz0);
	coren(Azim[n], Plun[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia1, Da1);
	bedcor(Strik[n], Dip[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia1+2, Da1+2);
	magcar(1., Ia2[1], Da2[1], &dumx0, &dumy0, &dumz0);
	coren(Azim[n], Plun[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia2, Da2);
	bedcor(Strik[n], Dip[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia2+2, Da2+2);
}

/* core_stg -- transfer GTC on core frame to in situ & tilted frames */

void core_stg(n, ux, uy, uz)
int n;
double ux, uy, uz;
{
	core(Azim[n], Plun[n], ux, uy, uz, Ux+1, Uy+1, Uz+1);
	carmag(Ux[1], Uy[1], Uz[1], &dumm, Ip+1, Dp+1);
	bedcor(Strik[n], Dip[n], Ux[1], Uy[1], Uz[1], Ux+2, Uy+2, Uz+2);
	carmag(Ux[2], Uy[2], Uz[2], &dumm, Ip+2, Dp+2);
	magcar(1., Ia1[0], Da1[0], &dumx0, &dumy0, &dumz0);
	core(Azim[n], Plun[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia1+1, Da1+1);
	bedcor(Strik[n], Dip[n], dumx1, dumy1, dumz1, &dumx0, &dumy0, &dumz0);
	carmag(dumx0, dumy0, dumz0, &dumm, Ia1+2, Da1+2);
	magcar(1., Ia2[0], Da2[0], &dumx0, &dumy0, &dumz0);
	core(Azim[n], Plun[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia2+1, Da2+1);
	bedcor(Strik[n], Dip[n], dumx1, dumy1, dumz1, &dumx0, &dumy0, &dumz0);
	carmag(dumx0, dumy0, dumz0, &dumm, Ia2+2, Da2+2);
}

/* tilt_csg -- transfer GTC on tilted frame to core & in situ frames */

void tilt_csg(n, ux, uy, uz)
int n;
double ux, uy, uz;
{
	bedcorn(Strik[n], Dip[n], ux, uy, uz, Ux+1, Uy+1, Uz+1);
	carmag(Ux[1], Uy[1], Uz[1], &dumm, Ip+1, Dp+1);
	coren(Azim[n], Plun[n], Ux[1], Uy[1], Uz[1], Ux, Uy, Uz);
	carmag(Ux[0], Uy[0], Uz[0], &dumm, Ip, Dp);
	magcar(1., Ia1[2], Da1[2], &dumx0, &dumy0, &dumz0);
	bedcorn(Strik[n], Dip[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia1+1, Da1+1);
	coren(Azim[n], Plun[n], dumx1, dumy1, dumz1, &dumx0, &dumy0, &dumz0);
	carmag(dumx0, dumy0, dumz0, &dumm, Ia1, Da1);
	magcar(1., Ia2[2], Da2[2], &dumx0, &dumy0, &dumz0);
	bedcorn(Strik[n], Dip[n], dumx0, dumy0, dumz0, &dumx1, &dumy1, &dumz1);
	carmag(dumx1, dumy1, dumz1, &dumm, Ia2+1, Da2+1);
	coren(Azim[n], Plun[n], dumx1, dumy1, dumz1, &dumx0, &dumy0, &dumz0);
	carmag(dumx0, dumy0, dumz0, &dumm, Ia2, Da2);
}

/* pdmg_gtc_parm -- confined section on GTC */

void pdmg_gtc_parm(f)
int f;
{
	int sign;
	double delta, cosd, sind, g1i, g1d, gg1i, gg1d, g2i, g2d, gg2i, gg2d, \
		dumm, dumi, dumd;

	delta = 2.*Mad;
	cosd = cos(delta/RAD);   sind = sin(delta/RAD);
	seek_g(Ip[f], Dp[f], Is[f][I2], Ds[f][I2], &g2i, &g2d);
	seek_g(Ip[f], Dp[f], Is[f][I1], Ds[f][I1], &g1i, &g1d);
	vecumulv(g1i, g1d, g2i, g2d, &dumi, &dumd);
	if ( vecumuls(dumi, dumd, Ip[f], Dp[f]) > 0.) sign = 1;
	else sign = -1;
	if (sign > 0){
		vecumulv(g1i, g1d, Ip[f], Dp[f], &gg1i, &gg1d);
		vecumulv(g2i, g2d, Ip[f], Dp[f], &gg2i, &gg2d);
	}
	else {
		vecumulv(-g1i, g1d-180., Ip[f], Dp[f], &gg1i, &gg1d);
		vecumulv(-g2i, g2d-180., Ip[f], Dp[f], &gg2i, &gg2d);
	}
	vecadd(cosd, g2i, g2d, sind, gg2i, gg2d, &dumm, Ia1+f, Da1+f);
	vecadd(-cosd, g1i, g1d, sind, gg1i, gg1d, &dumm, Ia2+f, Da2+f);
}

/* seek_g -- equation (20) of McFadden & McElhinny 1988 */

void seek_g(incp, decp, inc, dec, incg, decg)
double incp, decp, inc, dec, *incg, *decg;
{
	double tau, rho, p, q, r, u, v, w, x, y, z, dumm;

	tau = vecumuls(incp, decp, inc, dec);
	rho = sqrt(1. - tau*tau);
	magcar(1., incp, decp, &p, &q, &r);
	magcar(1., inc, dec, &u, &v, &w);
	x = (u - tau*p)/rho;
	y = (v - tau*q)/rho;
	z = (w - tau*r)/rho;
	carmag(x, y, z, &dumm, incg, decg);
}
