#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pdemag.h"

extern char **Sampl, Filed[WFILE];
extern unsigned int Opt;
extern int Frame, Nstep, I1, I2;
extern double *Azim, *Plun, *Strik, *Dip, **Xs, **Ys, **Zs, Xzj1[3], Xzj2[3], Yzjyr1[3], \
		Yzjyr2[3], Yzjzr1[3], Yzjzr2[3], Factzj[3], Ux[3], Uy[3], Uz[3], Iu[3], Du[3], \
		Madc, Mad, Theta;

static int nr;
static double dumm;

/* pdmg_line -- linear regression by the Principal component analysis on Zijderveld */

void pdmg_line(n)
int n;
{
	int anchor = 0;
	double xb, yb, zb, incb, decb;

	if (Opt & ANCHOR_BIT) anchor = 1;
	nr = I2 - I1 + 1;
	if ( (nr == 1) && !(Opt & ANCHOR_BIT) ){
		Opt |= ANCHOR_BIT;   anchor = 1;
	}
	pcalin(Xs[Frame]+I1, Ys[Frame]+I1, Zs[Frame]+I1, nr, anchor, &xb, &yb, &zb, \
		Ux+Frame, Uy+Frame, Uz+Frame, &Mad);
	carmag(Ux[Frame], Uy[Frame], Uz[Frame], &dumm, Iu+Frame, Du+Frame);
	set_zjpcaline(Frame, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	if (Frame == SITU) situ_coretilt(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame == CORE) core_situtilt(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame == TILT) tilt_coresitu(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	if ( !(Opt & ANCHOR_BIT) ){
		carmag(xb, yb, zb, &dumm, &incb, &decb);
		Theta = angdif(1., incb, decb, 1., Iu[Frame], Du[Frame]);
	}
}

/* pdmg_linea -- automatic PCA simplified from Kirshvink (1980)   061218 */

int pdmg_linea(n)
int n;
{
	int c, i1, i2, origin = 1, anchor = 0;
	double xb, yb, zb, incb, decb;

	nr = I2 - I1 + 1;
	if (nr < 2) error("nr=1 at pdmg_linea!\n");
	else if (nr == 2){
		if (Opt & ANCHOR_BIT) anchor = 1;
		else if (!(Opt & ORIGIN_BIT)) Opt |= ORIGIN_BIT;
	}
	else {
		if (!(Opt & ORIGIN_BIT)) origin = 0;
		if (Opt & ANCHOR_BIT) anchor = 1;
	}
	c = pcalina(Xs[Frame]+I1, Ys[Frame]+I1, Zs[Frame]+I1, nr, Madc, origin, anchor, \
		&xb, &yb, &zb, Ux+Frame, Uy+Frame, Uz+Frame, &i1, &i2, &Mad);
	if (c == ZERO) return ZERO;
	I1 = i1;  I2 = i2;
	carmag(Ux[Frame], Uy[Frame], Uz[Frame], &dumm, Iu+Frame, Du+Frame);
	set_zjpcaline(Frame, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	if (Frame == SITU) situ_coretilt(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame==CORE) core_situtilt(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	else if (Frame==TILT) tilt_coresitu(n, xb, yb, zb, Ux[Frame], Uy[Frame], Uz[Frame]);
	if ( !(Opt & ANCHOR_BIT) ){
		carmag(xb, yb, zb, &dumm, &incb, &decb);
		Theta = angdif(1., incb, decb, 1., Iu[Frame], Du[Frame]);
	}
	return +1;
}

/* situ_coretilt -- transfer in situ PCA to core & tilted frames */

void situ_coretilt(n, xb, yb, zb, ux, uy, uz)
int n;
double xb, yb, zb, ux, uy, uz;
{
	double xbc, ybc, zbc, xbt, ybt, zbt;

	coren(Azim[n], Plun[n], xb, yb, zb, &xbc, &ybc, &zbc);
	coren(Azim[n], Plun[n], ux, uy, uz, Ux, Uy, Uz);
	carmag(Ux[0], Uy[0], Uz[0], &dumm, Iu, Du);
	bedcor(Strik[n], Dip[n], xb, yb, zb, &xbt, &ybt, &zbt);
	bedcor(Strik[n], Dip[n], ux, uy, uz, Ux+2, Uy+2, Uz+2);
	carmag(Ux[2], Uy[2], Uz[2], &dumm, Iu+2, Du+2);
	set_zjpcaline(CORE, xbc, ybc, zbc, Ux[0], Uy[0], Uz[0]);
	set_zjpcaline(TILT, xbt, ybt, zbt, Ux[2], Uy[2], Uz[2]);
}

/* core_situtilt -- transfer PCA results on core frame to in situ & tilted frames */

void core_situtilt(n, xb, yb, zb, ux, uy, uz)
int n;
double xb, yb, zb, ux, uy, uz;
{
	double xbs, ybs, zbs, xbt, ybt, zbt;

	core(Azim[n], Plun[n], xb, yb, zb, &xbs, &ybs, &zbs);
	core(Azim[n], Plun[n], ux, uy, uz, Ux+1, Uy+1, Uz+1);
	carmag(Ux[1], Uy[1], Uz[1], &dumm, Iu+1, Du+1);
	bedcor(Strik[n], Dip[n], xbs, ybs, zbs, &xbt, &ybt, &zbt);
	bedcor(Strik[n], Dip[n], Ux[1], Uy[1], Uz[1], Ux+2, Uy+2, Uz+2);
	carmag(Ux[2], Uy[2], Uz[2], &dumm, Iu+2, Du+2);
	set_zjpcaline(SITU, xbs, ybs, zbs, Ux[1], Uy[1], Uz[1]);
	set_zjpcaline(TILT, xbt, ybt, zbt, Ux[2], Uy[2], Uz[2]);
}

/* tilt_coresitu -- transfer PCA results on tilted frame to core & in situ frames */

void tilt_coresitu(n, xb, yb, zb, ux, uy, uz)
int n;
double xb, yb, zb, ux, uy, uz;
{
	double xbc, ybc, zbc, xbs, ybs, zbs;

	bedcorn(Strik[n], Dip[n], xb, yb, zb, &xbs, &ybs, &zbs);
	bedcorn(Strik[n], Dip[n], ux, uy, uz, Ux+1, Uy+1, Uz+1);
	carmag(Ux[1], Uy[1], Uz[1], &dumm, Iu+1, Du+1);
	coren(Azim[n], Plun[n], xbs, ybs, zbs, &xbc, &ybc, &zbc);
	coren(Azim[n], Plun[n], Ux[1], Uy[1], Uz[1], Ux, Uy, Uz);
	carmag(Ux[0], Uy[0], Uz[0], &dumm, Iu, Du);
	set_zjpcaline(CORE, xbc, ybc, zbc, Ux[0], Uy[0], Uz[0]);
	set_zjpcaline(SITU, xbs, ybs, zbs, Ux[1], Uy[1], Uz[1]);
}

/* set_zjpcaline -- set edge points to draw a regression line on Zj plot
		(Xzj1,Yzjyr1)-(Xzj2,Yzjyr2) -- projection to x-y plane (y = a + bx)
		(Xzj1,Yzjzr1)-(Xzj2,Yzjzr2) -- projection to x-z plane (z = c + dx)
	args: f -- number of frame coordinate
*/

void set_zjpcaline(f, xb, yb, zb, ux, uy, uz)
int f;
double xb, yb, zb, ux, uy, uz;
{
	double a, b, c, d;

	b = uy/ux;   a = (yb - b*xb)*Factzj[f];
	d = uz/ux;   c = (zb - d*xb)*Factzj[f];
	Yzjyr1[f] = a + b*Xzj1[f];   Yzjyr2[f] = a + b*Xzj2[f];
	Yzjzr1[f] = c + d*Xzj1[f];   Yzjzr2[f] = c + d*Xzj2[f];
}
