#include <math.h>
#include "pdirec.h"

extern unsigned int Frm;
extern int Ns, Nsg, Nst, Nstg, *Id, *Rjc;
extern double *Slt, *Sln, **Is, **Ds, *Xs, *Ys, *Zs, *Dlt, *Vlt, *Vln, *Dltv, Im, Dm, R, K, \
		A95, Asd, Vltm, Vlnm, Vltmr, Vlnmr, dP, dM, Asdv, Imeq, Dmeq, Imeqr;

/* pdrc_statf -- Fisher statisitcs of the site */

void pdrc_statf(n)
int n;
{
	int i, k = 0;
	double x, y, z, m, cosa, cosi, dumm;

	if (Ns <= 1){
		Im = Is[Frm][Id[0]];   Dm = Ds[Frm][Id[0]];
		A95 = -1.;   R = 1.;   K = -1.;   Asd = -1.;
		Vltm = Vlt[0];   Vlnm = Vln[0];
		Vltmr = -Vltm;   Vlnmr = Vlnm + 180.;   checklon(&Vlnmr, 0);
		dP = -1.;   dM = -1.;   Asdv = -1.;
		Nst = 1;   Dlt[0] = 0.;   Dltv[0] = 0.;
		return;
	}
	x = y = z = 0.;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 1) continue;
		x += Xs[i];   y += Ys[i];   z += Zs[i];
		k++;
 	}
	if ((Nst = k) == 1) error("pdrc_statf: error of Nst=1!");
	m = k;
	carmag(x, y, z, &R, &Im, &Dm);
	K = (m - 1.)/(m - R);
	cosa = 1.  -  (m - R)/R*(pow(20., 1./(m-1.)) - 1.);
	if (cosa > 1.) A95 = 0.;
	else if (cosa < -1.) A95 = 180.;
	else A95 = acos(cosa)*RAD;
	fldvgp(Slt[n], Sln[n], Im, Dm, &Vltm, &Vlnm);
	Vltmr = -Vltm;   Vlnmr = Vlnm + 180.;   checklon(&Vlnmr, 0);
	cosi = cos(Im/RAD);   dumm = 1. + 3.*cosi*cosi;
	dP = 2.*A95/dumm;   dM = 2.*A95/sqrt(dumm);
	Imeq = Im;   Dmeq = 360. - Dm;   Imeqr = -Im;
	Asd = Asdv = 0.;
	for (i = 0; i < Ns; i++){
		Dlt[i] = angdif(1., Is[Frm][Id[i]], Ds[Frm][Id[i]], 1., Im, Dm);
		Dltv[i] = angdif(1., Vlt[i], Vln[i], 1., Vltm, Vlnm);
		if (Rjc[i] == 1) continue;
		Asd += Dlt[i]*Dlt[i];   Asdv += Dltv[i]*Dltv[i];
	}
	Asd = sqrt(Asd/(m - 1.));   Asdv = sqrt(Asdv/(m - 1.));
}

/* pdrc_rejectf -- reject the farthest data from the present Fisher statistics */

void pdrc_rejectf(n)
int n;
{
	int i, k = 0, imax = 0;
	double dltmax = 0.;

	if (Nst <= 2) return;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 1) continue;
		if (Dlt[i] >= dltmax){
			dltmax = Dlt[i];
			imax = i;
		}
		k++;
	}
	Rjc[imax] = 1;
	Nst = k - 1;
}

/* pdrc_restorf -- restore the nearest data to the present Fisher statistics */

void pdrc_restorf(n)
int n;
{
	int i, k = 0, imin = 0;
	double dltmin = 180.;

	if (Nst >= Ns) return;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 0){
			k++;
			continue;
		}
		if (Dlt[i] <= dltmin){
			dltmin = Dlt[i];
			imin = i;
		}
	}
	Rjc[imin] = 0;
	Nst = k + 1;
}
