#include <math.h>
#include "pdirec.h"

#define EPS	1.e-10 /* Often over 100 iterations when EPS=1.e-20 */
#define ITMAX	200 /* Stop iteration when over ITMAX. Often stopped when ITMAX=100 */
#define CONTIT0	"Initial of iteration was (I=%.1f,D=%.1f). To change, hit \"I\" command.\n"
#define CONTIT1	"Iteration was stopped at %d times!\n"

extern unsigned int Opt, Frm;
extern int Ns, Nsg, Nst, Nstg, *Id, *Idg, *Rjc, *Rjcg;
extern double *Slt, *Sln, **Is, **Ds, **Ip, **Dp, **Ia1, **Da1, **Ia2, **Da2;
extern double *Xs, *Ys, *Zs, *Dlt, *Dltg, *Vlt, *Vln, *Xp, *Yp, *Zp, *Ig, *Dg, \
		*Vltg, *Vlng, *Vltgr, *Vlngr, *Dltv, *Dltvg, Im, Dm, R, K, A95, Asd, \
		Vltm, Vlnm, Vltmr, Vlnmr, dP, dM, Asdv;
extern double *Ieqg, *Deqg, *Ieqgr, Imeq, Dmeq, Imeqr;

static double *xg, *yg, *zg, *xg0, *yg0, *zg0, *xa1, *ya1, *za1, *xa2, *ya2, *za2, *cosaa;

/* pdrc_statg -- statistics by McFadden & McElhinny to the combined data of line and plane
	NOTE: ASD is calculated by replacing total number by Nst+Nstg/2, because otherwise
		the ASD becomes too small compared to a95 by MM88 (I can not prove its
		statistical significance, though)
*/

void pdrc_statg(n, inci, deci)
int n;
double inci, deci;
{
	int i, j, k = 0, l = 0, it = 0;
	double x, y, z, xi, yi, zi, *dlt, mm, nn, ndsh, cosa, cosi, dumm;

	if (Ns == 0 && Nsg == 1){
		do_one_gtc();   return;
	}
	if (Ns==0 && Nsg==2){
		do_two_gtc(n);   return;
	}
	xg = mallocvd(Nsg);   yg = mallocvd(Nsg);   zg = mallocvd(Nsg);
	xg0 = mallocvd(Nsg);   yg0 = mallocvd(Nsg);   zg0 = mallocvd(Nsg);
	xa1 = mallocvd(Nsg);   ya1 = mallocvd(Nsg);   za1 = mallocvd(Nsg);
	xa2 = mallocvd(Nsg);   ya2 = mallocvd(Nsg);   za2 = mallocvd(Nsg);
	dlt = mallocvd(Nsg);   cosaa = mallocvd(Nsg);
	x = y = z = 0.;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 1) continue;
		x += Xs[i];   y += Ys[i];   z += Zs[i];
		k++;
	}
	Nst = k;
	if (Ns == 0 || Nst == 0){
		Opt |= IT_INITIAL_BIT;
		magcar(1., inci, deci, &xi, &yi, &zi);
		x = xi;   y = yi;   z = zi;
		fprintf(stdout, CONTIT0, inci, deci);
	}
	for (j = 0; j < Nsg; j++){
		magcar(1., Ia1[Frm][Idg[j]], Da1[Frm][Idg[j]], xa1+j, ya1+j, za1+j);
		magcar(1., Ia2[Frm][Idg[j]], Da2[Frm][Idg[j]], xa2+j, ya2+j, za2+j);
		cosaa[j] = xa1[j]*xa2[j] + ya1[j]*ya2[j] + za1[j]*za2[j];
		dlt[j] = 1.;
	}
	for (j = 0; j < Nsg; j++){
		if (Rjcg[j] == 1) continue;
		add_g(j, &x, &y, &z);
		l++;
	}
	Nstg = l;
	if (Ns ==0 || Nst == 0){
		x -= xi;   y -= yi;   z -= zi;
	}
	while (check_dlt(dlt) == 0){
		for (j = 0; j < Nsg; j++){
			if (Rjcg[j] == 1) continue;
			xg0[j] = xg[j];   yg0[j] = yg[j];   zg0[j] = zg[j];
			dlt[j] = recal_g(j, &x, &y, &z);
		}
		it++;
		if (it > ITMAX){
			fprintf(stdout, CONTIT1, ITMAX);
			break;
		}
	}
	carmag(x, y, z, &R, &Im, &Dm);
	mm = Nst;   nn = Nstg;
	K = 0.5*(2.*mm + nn - 2.)/(mm + nn - R);
	ndsh = mm + nn/2.;
	cosa = 1. - (ndsh - 1.)/(K*R)*(pow(20., 1./(ndsh-1.)) - 1.);
	if (cosa > 1.) A95 = 0.;
	else if (cosa < -1.) A95 = 180.;
	else A95 = acos(cosa)*RAD;
	fldvgp(Slt[n], Sln[n], Im, Dm, &Vltm, &Vlnm);
	Vltmr = -Vltm;   Vlnmr = Vlnm + 180.;   checklon(&Vlnmr, 0);
	cosi = cos(Im/RAD);   dumm = 1. + 3.*cosi*cosi;
	dP = 2.*A95/dumm;   dM = 2.*A95/sqrt(dumm);
	Imeq = Im;   Dmeq = 360. - Dm;   Imeqr = -Im;
	Asd = Asdv = 0.;
	for (i = 0; i < Ns; i++){
		Dlt[i] = angdif(1., Is[Frm][Id[i]], Ds[Frm][Id[i]], 1., Im, Dm);
		Dltv[i] = angdif(1., Vlt[i], Vln[i], 1., Vltm, Vlnm);
		if (Rjc[i] == 1) continue;
		Asd += Dlt[i]*Dlt[i];   Asdv += Dltv[i]*Dltv[i];
	}
	for (j = 0; j < Nsg; j++){
		if (Rjcg[j] == 1){
			Ig[j] = Dg[j] = Dltg[j] = Dltvg[j] = 0.;
			continue;
		}
		carmag(xg[j], yg[j], zg[j], &dumm, Ig+j, Dg+j);
		Ieqg[j] = Ig[j];   Deqg[j] = 360. - Dg[j];   Ieqgr[j] = -Ieqg[j];
		fldvgp(Slt[n], Sln[n], Ig[j], Dg[j], Vltg+j, Vlng+j);
		Vltgr[j] = -Vltg[j];   Vlngr[j] = Vlng[j] + 180.;   checklon(Vlngr+j, 0);
		Dltg[j] = angdif(1., Ig[j], Dg[j], 1., Im, Dm);
		Dltvg[j] = angdif(1., Vltg[j], Vlng[j], 1., Vltm, Vlnm);
		Asd += Dltg[j]*Dltg[j];   Asdv += Dltvg[j]*Dltvg[j];
	}
	Asd = sqrt(Asd/(ndsh - 1.));   Asdv = sqrt(Asdv/(ndsh - 1.));
	freevd(xg); freevd(yg); freevd(zg); freevd(xg0); freevd(yg0); freevd(zg0);
	freevd(xa1); freevd(ya1); freevd(za1); freevd(xa2); freevd(ya2); freevd(za2);
	freevd(dlt); freevd(cosaa);
}

/* add_g -- add a point G on the j th plane to the present data set which starts
	from the line data only (Ns) to the final combined dataset (Ns+Nsg)
*/

void add_g(j, x, y, z)
int j;
double *x, *y, *z;
{
	double u, v, w, t, r, dumm;

	dumm = sqrt((*x)*(*x) + (*y)*(*y) + (*z)*(*z));
	u = *x/dumm;   v = *y/dumm;   w = *z/dumm;
	t = u*Xp[j] + v*Yp[j] + w*Zp[j];
	r = sqrt(1. - t*t);
	xg[j] = (u - t*Xp[j])/r;   yg[j] = (v - t*Yp[j])/r;   zg[j] = (w - t*Zp[j])/r;
	check_ag(j);
	*x += xg[j];   *y += yg[j];   *z += zg[j];
}

/* recal_g -- adjust a point G on the j th plane to best fit to the combined data
	returns 1 - cos(theta), where theta is the angle between new and previous Gs
*/

double recal_g(j, x, y, z)
int j;
double *x, *y, *z;
{
	double u, v, w, t, r, dumm;

	dumm = sqrt((*x)*(*x) + (*y)*(*y) + (*z)*(*z));
	u = *x/dumm;   v = *y/dumm;   w = *z/dumm;
	t = u*Xp[j] + v*Yp[j] + w*Zp[j];
	r = sqrt(1. - t*t);
	xg[j] = (u - t*Xp[j])/r;   yg[j] = (v - t*Yp[j])/r;   zg[j] = (w - t*Zp[j])/r;
	check_ag(j);
	*x += xg[j] - xg0[j];   *y += yg[j] - yg0[j];   *z += zg[j] - zg0[j];
	return 1. - xg0[j]*xg[j] - yg0[j]*yg[j] - zg0[j]*zg[j];
}

/* check_ag -- check the the position of j th G whether it is within the confined section, 
	if not, replace it with the nearest A point (A1 or A2 of the j th GTC plane)
*/

void check_ag(j)
int j;
{
	int imin = 0;
	double cosmin = cosaa[j], cosga2, cosga1;

	cosga2 = xg[j]*xa2[j] + yg[j]*ya2[j] + zg[j]*za2[j];
	cosga1 = xg[j]*xa1[j] + yg[j]*ya1[j] + zg[j]*za1[j];
	if (cosga2 < cosmin){
		cosmin = cosga2;
		imin = 1;
	}
	if (cosga1 < cosmin){
		cosmin = cosga1;
		imin = 2;
	}
	switch (imin){
	case 0:
		break;
	case 1:
		xg[j] = xa1[j];   yg[j] = ya1[j];   zg[j] = za1[j];
		break;
	case 2:
		xg[j] = xa2[j];   yg[j] = ya2[j];   zg[j] = za2[j];
		break;
	default:
		error("check_ag: error in imin!");
	}
}

/* check_dlt -- check whether all G points stablized by comparing dlt[j] one by one
	to the pre-defined EPS.  pdrc_statg() stops iteration when all dlt[] < EPS
*/

int check_dlt(dlt)
double *dlt;
{
	int flag = 1, j;

	for (j = 0; j < Nsg; j++){
		if (Rjcg[j] == 1) continue;
		if (dlt[j] > EPS){
			flag = 0;
			break;
		}
	}
	return(flag);
}

/* pdrc_rejectg -- reject the farthest sample in the combined Fisher & GTC data for site n */

void pdrc_rejectg(n)
int n;
{
	int i, j, k = 0, l = 0, imax = 0, jmax = 0;
	double dltmax = 0., dltmaxg = 0.;

	if (Nst==0 && Nstg<=3) return;
	if (Nst==1 && Nstg==1) return;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 1) continue;
 		if (Dlt[i] >= dltmax){
			dltmax = Dlt[i];
			imax = i;
		}
		k++;
	}
	for (j = 0; j < Nsg; j++){
		if (Rjcg[j] == 1) continue;
 		if (Dltg[j] >= dltmaxg){
			dltmaxg = Dltg[j];
			jmax = j;
		}
		l++;
	}
	if (dltmax >= dltmaxg){
		if (Nst==1 && Nstg==2){
			fprintf(stdout, "Can not delete the last line data!\n");
			return;
		}
		Rjc[imax] = 1;
		Nst = k - 1;
	}
	else {
		Rjcg[jmax] = 1;
		Nstg = l - 1;
	}
}

/* pdrc_restorg -- restore the nearest sample in the combined data for site n */

void pdrc_restorg(n)
int n;
{
	int i, j, k = 0, l = 0, imin = 0, jmin = 0;
	double dltmin = 180., dltming = 180.;

	if (Nst+Nstg >= Ns+Nsg) return;
	for (i = 0; i < Ns; i++){
		if (Rjc[i] == 0){
			k++;
			continue;
		}
		if (Dlt[i] <= dltmin){
			dltmin = Dlt[i];
			imin = i;
		}
	}
	for (j = 0; j < Nsg; j++){
		if (Rjcg[j] == 0){
			l++;
			continue;
		}
		if (Dltg[j] <= dltming){
			dltming = Dltg[j];
			jmin = j;
		}
	}
	if (dltmin <= dltming){
		Rjc[imin] = 0;
		Nst = k + 1;
	}
	else {
		Rjcg[jmin] = 0;
		Nstg = l + 1;
	}
}

/* do_one_gtc -- set up Im, Dm, ... when only data is one GTC (Ns=0 & Nsg=1) */

void do_one_gtc()
{
	Im = 0.;   Dm = 0.;
	A95 = -1.;   R = -1.;   K = -1.;   Asd = -1.;
	Vltm = 0.;   Vlnm = 0.;   dP = -1.;   dM = -1.;   Asdv = -1.;
	Ig[0] = 0.;   Dg[0] = 0.;   Dltg[0] = 0.;   Dltvg[0] = 0.;
}

/* do_two_gtc -- cross point when only data are two GTCs (Ns=0 & Nsg=2) */

void do_two_gtc(n)
int n;
{
	int j = 0, jc = 0;
	double inc, dec, Ipa, Dpa, Ipb, dPb, Ia1a, Da1a, Ia2a, Da2a, Ia1b, Da1b, Ia2b, Da2b, \
		cosAA, cosA1, cosA2;

	Ipa = Ip[Frm][Idg[0]];   Dpa = Dp[Frm][Idg[0]];
	Ia1a = Ia1[Frm][Idg[0]];   Da1a = Da1[Frm][Idg[0]];
	Ia2a = Ia2[Frm][Idg[0]];   Da2a = Da2[Frm][Idg[0]];
	Ig[0] = 0.;   Dg[0] = 0.;   Dltg[0] = 0.;
	Ipb = Ip[Frm][Idg[1]];   dPb = Dp[Frm][Idg[1]];
	Ia1b = Ia1[Frm][Idg[1]];   Da1b = Da1[Frm][Idg[1]];
	Ia2b = Ia2[Frm][Idg[1]];   Da2b = Da2[Frm][Idg[1]];
	Ig[1] = 0.;   Dg[1] = 0.;   Dltg[1] = 0.;
	vecumulv(Ipa, Dpa, Ipb, dPb, &inc, &dec);
	cosAA = vecumuls(Ia1a, Da1a, Ia2a, Da2a);
	cosA1 = vecumuls(Ia1a, Da1a, inc, dec);
	cosA2 = vecumuls(Ia2a, Da2a, inc, dec);
	if ((cosAA>cosA1) || (cosAA>cosA2)){
		inc = -inc;
		dec = dec + 180.;   checklon(&dec, 0);
	}
	Im = inc;   Dm = dec;
	fldvgp(Slt[n], Sln[n], Im, Dm, &Vltm, &Vlnm);
	Imeq = Im;   Dmeq = 360. - Dm;   Imeqr = -Imeq;
	A95 = -1.;   R = -1.;   K = -1;   Asd = -1.;
	Vltmr = -Vltm;   Vlnmr = Vlnm + 180.;   checklon(&Vlnmr, 0);
	dP = -1.;   dM = -1.;   Asdv = -1.;
}
#undef EPS
#undef ITMAX
#undef CONTIT0
#undef CONTIT1
