/* pi_dat -- read data from a file and stores as a series in Magd[], Ivncd[], ...
	Amendment: Chid[j]=0 is set when no chi data is available   051919
	Changes for V.1.2: introduced memory allocation by mallocms(), mallocvd(), etc.
		to all array vriables.  Hence there is no limit to number of data lines
		in the original file or number of steps, and so on.
	External variables:
		see pinte.c
	Internal variables:
		i -- counts number of samples
		j -- counts number of data lines in series through samples
		k -- counter of zero steps for each sample
		l -- counter of "+" steps for each sample
		nstepmax -- used as dimension of all step related variables
		ptmax -- used as dimension of all variables related to pTRM test
		mdmax -- used as dimension of all variables related to MD pTRM tail check
	Return:
		number of data lines in the original data file
*/

#include <string.h>
#include <stdlib.h>
#include "pinte.h"

extern char **Sampl, **Stepd;
extern unsigned int Methd;
extern int Nd, Nptr, Nmdtl, *Idsmpl, *Idpt1, *Idpt2, *Idmd, *Ia;
extern double *Azim, *Plun, *Magd, *Incd, *Decd, *Chid, *Step, *Xn, *Yn, *Zn, *Chin, \
        *Xmd, *Ymd, *Zmd, *Chim, *Xt, *Yt, *Zt, *Chit, *Xp, *Yp, *Zp, *Chip, \
	*Magn, *Incn, *Decn, *Magn2, *Incn2, *Decn2, *Magmd, *Incmd, *Decmd, \
	*Magt, *Inct, *Dect, *Magp, *Incp, *Decp;
extern double *Ienn, *Denn, *Ienr, *Iemn, *Demn, *Iemr, *Ietn, *Detn, *Ietr, \
        *Iepn, *Depn, *Iepr, *Xzn, *Yzn, *Zzn, *Xzm, *Yzm, *Zzm, *Xzt, *Yzt, *Zzt, \
        *Xzp, *Yzp, *Zzp, *Ycn, *Yct, *Ycp, *Ycm, *Xa, *Xac, *Xam, *Xacm, *Ya, *Yam, \
	*Xap, *Xacp, *Yap, *Incs, *Decs, *Xs, *Ys, *Zs;

int pi_dat(fp)
FILE *fp;
{

	char s1[30], s2[30], s3[30], s4[30], s5[30];
	int n, i = 0, j = 0, k = 0, nstepmax = 0, l = 0, ptmax = 0, mdmax = 0, nline;

	while ((n = get_s(fp, 5, s1, s2, s3, s4, s5)) != EOF){
		if (n==0 || s1[0]=='#' || s1[0]=='%' || strcmp("$$",s1)==0) continue;
		if (strcmp("$", s1) == 0) i++;
		else j++;
	}
	Nd = i;   nline = j;
	pi_mem1(Nd, nline);
	i = 0;   j = 0;   rewind(fp);
	while ((n = get_s(fp, 5, s1, s2, s3, s4, s5)) != EOF){
		if (n == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp("$$", s1) == 0){
			if (stricmp("COE",s2)==0 || stricmp("ZI",s2)==0)
				Methd = COE;
			else if (stricmp("AITKEN",s2)==0 || stricmp("IZ",s2)==0)
				Methd = AITKEN;
			else if (stricmp("IZZI",s2)==0 || stricmp("TAUXE",s2)==0)
				Methd = IZZI;
			else
				error("Wrong method line!");
		}
		else if (strcmp("$", s1) == 0){
			if (n == 1) error("Error in $ line!");
			Idsmpl[i] = j;
			strcpy(Sampl[i], s2);
			if (n == 2){
				Azim[i] = Plun[i] = 0.;
			}
			else if (n == 3){
				Azim[i] = atof(s3);  Plun[i] = 0.;
			}
			else {
				Azim[i] = atof(s3);  Plun[i] = atof(s4);
			}
			count_step(&k, &l, &nstepmax, &ptmax, &mdmax);
			i++;
		}
		else {
			strcpy(Stepd[j], s1);
			Magd[j] = atof(s2);   Incd[j] = atof(s3);   Decd[j] = atof(s4);
			if (n == 4) Chid[j] = 0.;
			else Chid[j] = atof(s5);
			if (Stepd[j][0] != '+') k++;
			else l++;
			j++;
		}
	}
	Idsmpl[i] = j;
	count_step(&k, &l, &nstepmax, &ptmax, &mdmax);
	pi_mem2(nstepmax, ptmax, mdmax);   pi_mem3(nstepmax, ptmax, mdmax);
	return(nline);
}

/* count_step -- k & l count zero & "+" steps for each sample   092606,0201
		Due to difficulty of guessing number of steps (except Aitken),
		maximum numbers are used for memory allocation of step related variables
	COE (ZI):
		(max no of step) = (no of zero step)
		(max no of pTRM test) = (no of "+" step)
		(max no of MD pTRM tail test) = (no of zero step) - 1
	AITKEN (IZ):
		(no of step) = (no of "+" step) + 1
		(no of pTRM test) = (no of zero step) - (no of "+" step) - 1
	IZZI (Tauxe):
		(max no of step) = (no of zero step)
		(max no of pTRM test) = (no of "+" step)
		(max no of MD pTRM tail check) = (no of zero step) - 1
*/

void count_step(k, l, nstepmax, ptmax, mdmax)
int *k, *l, *nstepmax, *ptmax, *mdmax;
{
	int nstep, npt, nmd;

	switch(Methd){
	case COE:
		if (*k > *nstepmax) *nstepmax = *k;
		if (*l > *ptmax) *ptmax = *l;
		if (*k - 1 > *mdmax) *mdmax = *k - 1; 
		*k = *l = 0;
		break;
	case AITKEN:
		if (*l + 1 > *nstepmax) *nstepmax = *l + 1;
		if (*k - *l - 1 > *ptmax) *ptmax = *k - *l - 1;
		*k = *l = 0;
		break;
	case IZZI:
		if (*k > *nstepmax) *nstepmax = *k;
		if (*l > *ptmax) *ptmax = *l;
		if (*k - 1 > *mdmax) *mdmax = *k - 1;
		*k = *l = 0;
		break;
	default:
		error("Error within switch at count_step!");
		break;
	}
}

/* pi_mem1 -- memory allocation for Magd[], Incd[], Decd[]
	nd -- number of data or sample, nl -- number of data line in original file
*/

void pi_mem1(nd, nl)
int nd, nl;
{
	Stepd = mallocms(nl, WSTEP);
	Magd = mallocvd(nl);   Incd = mallocvd(nl);   Decd = mallocvd(nl);
	Chid = mallocvd(nl);
	Sampl = mallocms(nd, WSAMP);
	Idsmpl = mallocvi(nd);
	Azim = mallocvd(nd);   Plun = mallocvd(nd);
}

/* pi_mem2 -- memory allocation for step related variables
	ns, np, nm: maximum numbers of steps, pTRM tests, & MD pTRM tail checks
	seems to work even when np=0 or nm=0
*/

void pi_mem2(ns, np, nm)
int ns, np, nm;
{
	Step = mallocvd(ns);
	Xn = mallocvd(ns);   Yn = mallocvd(ns);   Zn = mallocvd(ns);
	Magn = mallocvd(ns);   Incn = mallocvd(ns);   Decn = mallocvd(ns);
	Chin = mallocvd(ns);

	Xt = mallocvd(ns);   Yt = mallocvd(ns);   Zt = mallocvd(ns);
	Magt = mallocvd(ns);   Inct = mallocvd(ns);   Dect = mallocvd(ns);
	Chit = mallocvd(ns);

	Idmd = mallocvi(nm);
	Magn2 = mallocvd(nm);   Incn2 = mallocvd(nm);   Decn2 = mallocvd(nm);
	Xmd = mallocvd(nm);   Ymd = mallocvd(nm);   Zmd = mallocvd(nm);
	Magmd = mallocvd(nm);   Incmd = mallocvd(nm);   Decmd = mallocvd(nm);
	Chim = mallocvd(nm);

	Idpt1 = mallocvi(np);   Idpt2 = mallocvi(np);
	Xp = mallocvd(np);   Yp = mallocvd(np);   Zp = mallocvd(np);
	Magp = mallocvd(np);   Incp = mallocvd(np);   Decp = mallocvd(np);
	Chip = mallocvd(np);
}

/* pi_mem3 -- memory allocation for all variables related to GMT plotting */

void pi_mem3(ns, np, nm)
int ns, np, nm;
{
	Ia = mallocvi(ns);
	Ienn = mallocvd(ns);   Denn = mallocvd(ns);   Ienr = mallocvd(ns);
	Ietn = mallocvd(ns);   Detn = mallocvd(ns);   Ietr = mallocvd(ns);
	Xzn = mallocvd(ns);   Yzn = mallocvd(ns);   Zzn = mallocvd(ns);
	Xzt = mallocvd(ns);   Yzt = mallocvd(ns);   Zzt = mallocvd(ns);
	Ycn = mallocvd(ns);   Yct = mallocvd(ns);
	Xa = mallocvd(ns);   Xac = mallocvd(ns);   Ya = mallocvd(ns);
	Incs = mallocvd(ns+np+nm);   Decs = mallocvd(ns+np+nm);
	Xs = mallocvd(ns+np+nm);   Ys = mallocvd(ns+np+nm);   Zs = mallocvd(ns+np+nm);

	Iepn = mallocvd(np);   Depn = mallocvd(np);   Iepr = mallocvd(np);
	Xzp = mallocvd(np);   Yzp = mallocvd(np);   Zzp = mallocvd(np);
	Ycp = mallocvd(np);
	Xap = mallocvd(np);   Xacp = mallocvd(np);   Yap = mallocvd(np);

	Iemn = mallocvd(nm);   Demn = mallocvd(nm);   Iemr = mallocvd(nm);
	Xzm = mallocvd(nm);   Yzm = mallocvd(nm);   Zzm = mallocvd(nm);
	Ycm = mallocvd(nm);
	Xam = mallocvd(nm);   Xacm = mallocvd(nm);   Yam = mallocvd(nm);
}

/* free_all -- free memory for all external array variables */

void free_all(nd, nl)
int nd, nl;
{
	freems(nl, Stepd);
	freevd(Magd);   freevd(Incd);   freevd(Decd);
	freevd(Chid);
	freems(nd, Sampl);
	freevi(Idsmpl);
	freevd(Azim);   freevd(Plun);

	freevd(Step);
	freevd(Xn);   freevd(Yn);   freevd(Zn);
	freevd(Magn);   freevd(Incn);   freevd(Decn);
	freevd(Chin);

	freevd(Xt);   freevd(Yt);   freevd(Zt);
	freevd(Magt);   freevd(Inct);   freevd(Dect);
	freevd(Chit);

	freevi(Idmd);
	freevd(Magn2);   freevd(Incn2);   freevd(Decn2);
	freevd(Xmd);   freevd(Ymd);   freevd(Zmd);
	freevd(Magmd);   freevd(Incmd);   freevd(Decmd);
	freevd(Chim);

	freevi(Idpt1);   freevi(Idpt2);
	freevd(Xp);   freevd(Yp);   freevd(Zp);
	freevd(Magp);   freevd(Incp);   freevd(Decp);
	freevd(Chip);

	freevi(Ia);
	freevd(Ienn);   freevd(Denn);   freevd(Ienr);
	freevd(Ietn);   freevd(Detn);   freevd(Ietr);
	freevd(Xzn);   freevd(Yzn);   freevd(Zzn);
	freevd(Xzt);   freevd(Yzt);   freevd(Zzt);
	freevd(Ycn);   freevd(Yct);
	freevd(Xa);   freevd(Xac);   freevd(Ya);
	freevd(Incs);   freevd(Decs);
	freevd(Xs);   freevd(Ys);   freevd(Zs);

	freevd(Iepn);   freevd(Depn);   freevd(Iepr);
	freevd(Xzp);   freevd(Yzp);   freevd(Zzp);
	freevd(Ycp);
	freevd(Xap);   freevd(Xacp);   freevd(Yap);

	freevd(Iemn);   freevd(Demn);   freevd(Iemr);
	freevd(Xzm);   freevd(Yzm);   freevd(Zzm);
	freevd(Ycm);
	freevd(Xam);   freevd(Xacm);   freevd(Yam);
}
