/* pi_ntp -- calculate vectors of NRM, TRM, MD pTRM tail check, and pTRM test for (i+1)th sample
	External variables:
		see pinte.c
	Args & return:
		nptr -- number of pTRM test (Nptr)
		nmdtl -- number of MD pTRM tail check (Nmdtl)
		i1, i2 -- range of regression I1 & I2
		Return -- number of steps
		(Purpose of using returns and args for external variables is to explicitly
		specify in main routine when these parameters are initialized)
	Internal variables:
		j -- (j+1)th line of data series
		k -- kth step (0th indicates NRM)
		l -- (l+1)th pTRM test
		m -- (m+1)th MD pTRM tail check
		trmdone -- introduced to allow pTRM test to the same step (by McClelland)
			not applicable to IZZI version
		izzi -- 1 & 0 indicate that step now is IZ & ZI protocols, respectively
		sign -- 1 & 0 indicate that each 1st step is I(+) & Z(0), respectively
		cp -- count no of pTRM tests (zero steps)
		xd, yd, zd, xdd, ydd, zdd -- dummy variables
*/

#include <ctype.h>
#include <stdlib.h>
#include "pinte.h"

/* #define ERRDAT	"Error in %s data of %s at %s\n" -- COMPILE ERROR!? */
#define ERRDAT	"Error of pTRM in %d+1th sample at %d+1th line.\n"

extern char **Sampl, **Stepd, *Id_mthd, Buf[WBUF];
extern int *Idsmpl, *Idpt1, *Idpt2, *Idmd, Methd;
extern double *Magd, *Incd, *Decd, *Chid, *Step, *Xn, *Yn, *Zn, *Chin, \
		*Xmd, *Ymd, *Zmd, *Chim, *Xt, *Yt, *Zt, *Chit, *Xp, *Yp, *Zp, *Chip, \
		*Magn, *Incn, *Decn, *Magn2, *Incn2, *Decn2, *Magmd, *Incmd, *Decmd, \
		*Magt, *Inct, *Dect, *Magp, *Incp, *Decp;
static int j, k, l, m, trmdone, izzi, sign, cp;
static double stepv, stepz, stept, xd, yd, zd, xdd, ydd, zdd;

int pi_ntp(i, nptr, nmdtl, i1, i2)
int i, *nptr, *nmdtl, *i1, *i2;
{
	switch(Methd) {
	case COE:
		return pi_ntpC(i, nptr, nmdtl, i1, i2);
		break;
	case AITKEN:
		return pi_ntpA(i, nptr, nmdtl, i1, i2);
		break;
	case IZZI:
		return pi_ntpIZZI(i, nptr, nmdtl, i1, i2);
		break;
	default:
		error("Error of Methd within switch at pi_ntp!");
		break;
	}
}

/* pi_ntpC -- Coe (ZI) type of Thellier paleointensity experiment */

int pi_ntpC(i, nptr, nmdtl, i1, i2)
int i, *nptr, *nmdtl, *i1, *i2;
{
	k = l = m = trmdone = 0;
	j = Idsmpl[i];
	if (Stepd[j][0] == '+') error("Should be data for room temp.!");
	stepv = Step[k] = atof(Stepd[j]);
	Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
	magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
	Chin[k] = Chid[j];
	Magt[k] = Inct[k] = Dect[k] = Xt[k] = Yt[k] = Zt[k] = 0.;
	Chit[k] = 0.;
	j++;
	while (j < Idsmpl[i+1]){
		if (isdigit(Stepd[j][0])){
			if ((stepz = atof(Stepd[j])) != stepv){
				k++;
				stepv = Step[k] = stepz;
				Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
				magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
				Chin[k] = Chid[j];
				trmdone = 0;
			}
			else {
				cal_mdtl(j, k, m);
				Chim[m] = Chid[j];
				Idmd[m] = k;
				m++;
			}
		}
		else if (Stepd[j][0] == '+'){
			stept = atof(Stepd[j]);
			if (stept == stepv && trmdone != 1){
				cal_trmC(j, k, Xt+k, Yt+k, Zt+k);
				carmag(Xt[k], Yt[k], Zt[k], Magt+k, Inct+k, Dect+k);
				Chit[k] = Chid[j];
				trmdone = 1;
			}
			else {
				cal_ptrmC(j, k, stept, Idpt1+l, Idpt2+l, Xp+l, Yp+l, Zp+l);
				carmag(Xp[l], Yp[l], Zp[l], Magp+l, Incp+l, Decp+l);
				Chip[l] = Chid[j];
				l++;
			}
		}
		j++;
	}
	*nptr = l;   *nmdtl = m;   *i1 = 0;   *i2 = k;
	return(k+1);
}

/* pi_ntpA -- Aitken's experimental procedure ot the Thellier method */

int pi_ntpA(i, nptr, nmdtl, i1, i2)
int i, *nptr, *nmdtl, *i1, *i2;
{
	k = l = m = trmdone = 0;
	j = Idsmpl[i];
	if (Stepd[j][0] == '+') error("Should be data for room temp.!");
	stepv = Step[k] = atof(Stepd[j]);
	Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
	magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
	Chin[k] = Chid[j];
	Magt[k] = Inct[k] = Dect[k] = Xt[k] = Yt[k] = Zt[k] = 0.;
	Chit[k] = 0.;
	j++;
	while (j < Idsmpl[i+1]){
		if (Stepd[j][0] == '+'){
			k++;
			if ((stept = atof(Stepd[j])) != stepv){
				stepv = Step[k] = stept;
				Chit[k] = Chid[j];
				cp = 0;   trmdone = 0;
			}
			else {
/* COMPILE ERROR!? 		fprintf(stderr, ERRDAT, Id_mthd[Methd], Sampl[i], Stepd[j]); */
				fprintf(stderr, ERRDAT, i, j);
				exit(1);
			}
		}
		else if (isdigit(Stepd[j][0])){
			stepz = atof(Stepd[j]);
			if (stepz == stepv && trmdone != 1) {
				Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
				magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
				cal_trmA(j, k, cp, Xt+k, Yt+k, Zt+k);
				carmag(Xt[k], Yt[k], Zt[k], Magt+k, Inct+k, Dect+k);
				Chin[k] = Chid[j];
				trmdone = 1;
			}
			else {
				cal_ptrmA(j, k, cp, stepz, Idpt1+l, Idpt2+l, Xp+l, Yp+l, Zp+l);
				carmag(Xp[l], Yp[l], Zp[l], Magp+l, Incp+l, Decp+l);
				Chip[l] = Chid[j];
				l++;
				cp++;
			}
		}
		j++;
	}
	*nptr = l;   *nmdtl = m;   *i1 = 0;   *i2 = k;
	return(k+1);
}

/* pi_ntpIZZI -- IZZI (Tauxe & Staudigel) version of the Thellier method
	multiple pTRM test (0 for IZ, + for ZI) is allowed, but multiple MD pTRM tail check
	and pTRM test to the same step are NOT allowed
*/

int pi_ntpIZZI(i, nptr, nmdtl, i1, i2)
int i, *nptr, *nmdtl, *i1, *i2;
{
	k = l = m = 0;
	izzi = sign = 1;
	j = Idsmpl[i];
	if (Stepd[j][0] == '+') error("Should be data for room temp.!");
	stepv = Step[k] = atof(Stepd[j]);
	Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
	magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
	Chin[k] = Chid[j];
	Magt[k] = Inct[k] = Dect[k] = Xt[k] = Yt[k] = Zt[k] = 0.;
	Chit[k] = 0.;
	j++;
	while (j < Idsmpl[i+1]){
		if (izzi == 1){ /* IZ steps */
			if (Stepd[j][0] == '+'){
				k++;
				if ((stept = atof(Stepd[j])) != stepv){
					stepv = Step[k] = stept;
					Chit[k] = Chid[j];
					cp = 0;   sign = 1;
				}
				else {
/* COMPILE ERROR!?			fprintf(stderr, ERRDAT, Id_mthd[Methd], Sampl[i], Stepd[j]); */
					fprintf(stderr, ERRDAT, i, j);
					exit(1);
				}
			}
			else if (isdigit(Stepd[j][0])){
				stepz = atof(Stepd[j]);
				if (stepz == stepv && sign == 0){
					cal_mdtl(j, k, m);
					Chim[m] = Chid[j];
					Idmd[m] = k;
					m++;
				}
				else if (stepz == stepv && sign == 1) {
					Magn[k] = Magd[j];  Incn[k] = Incd[j];  Decn[k] = Decd[j];
					magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
					cal_trmA(j, k, cp, Xt+k, Yt+k, Zt+k);
					carmag(Xt[k], Yt[k], Zt[k], Magt+k, Inct+k, Dect+k);
					Chin[k] = Chid[j];
					izzi = 0;
				}
				else if (stepz != stepv){
					cal_ptrmA(j, k, cp, stepz, Idpt1+l, Idpt2+l, Xp+l, Yp+l, Zp+l);
					carmag(Xp[l], Yp[l], Zp[l], Magp+l, Incp+l, Decp+l);
					Chip[l] = Chid[j];
					l++;   cp++;
				}
			}
		}
		else if (izzi == 0){ /* ZI steps */
			if (isdigit(Stepd[j][0])){
				if ((stepz = atof(Stepd[j])) != stepv){
					k++;
					stepv = Step[k] = stepz;
					Magn[k] = Magd[j];   Incn[k] = Incd[j];   Decn[k] = Decd[j];
					magcar(Magd[j], Incd[j], Decd[j], Xn+k, Yn+k, Zn+k);
					Chin[k] = Chid[j];
					sign = 0;
				}
			}
			else if (Stepd[j][0] == '+'){
				stept = atof(Stepd[j]);
				if (stept == stepv){
					cal_trmC(j, k, Xt+k, Yt+k, Zt+k);
					carmag(Xt[k], Yt[k], Zt[k], Magt+k, Inct+k, Dect+k);
					Chit[k] = Chid[j];
					izzi = 1;
				}
				else {
					cal_ptrmC(j, k, stept, Idpt1+l, Idpt2+l, Xp+l, Yp+l, Zp+l);
					carmag(Xp[l], Yp[l], Zp[l], Magp+l, Incp+l, Decp+l);
					Chip[l] = Chid[j];
					l++;
				}
			}
		}
		j++;
	}
	*nptr = l;   *nmdtl = m;   *i1 = 0;   *i2 = k;
	return(k+1);
}

/* cal_trmC -- TRM component for Coe (ZI) version */

void cal_trmC(j, k, xt, yt, zt)
int j, k;
double *xt, *yt, *zt;
{
	magcar(Magd[j], Incd[j], Decd[j], &xd, &yd, &zd);
	*xt = xd - Xn[k];   *yt = yd - Yn[k];   *zt = zd - Zn[k];
}

/* cal_ptrmC -- calculate pTRM component for Coe (ZI) version */

void cal_ptrmC(j, k, stept, id1, id2, xp, yp, zp)
int j, k, *id1, *id2;
double stept, *xp, *yp, *zp;
{
	*id1 = k;
	*id2 = seek_stept(stept, k);
	magcar(Magd[j], Incd[j], Decd[j], &xd, &yd, &zd);
	*xp = xd - Xn[k];   *yp = yd - Yn[k];   *zp = zd - Zn[k];
}

/* cal_trmA -- TRM component for Aitken (IZ) version */

void cal_trmA(j, k, cp, xt, yt, zt)
int j, k, cp;
double *xt, *yt, *zt;
{
	magcar(Magd[j-1-cp], Incd[j-1-cp], Decd[j-1-cp], &xd, &yd, &zd);
	*xt = xd - Xn[k];   *yt = yd - Yn[k];   *zt = zd - Zn[k];
}

/* cal_ptrmA -- calculate pTRM component for Aitken (IZ) version */

void cal_ptrmA(j, k, cp, stepz, id1, id2, xp, yp, zp)
int j, k, cp, *id1, *id2;
double stepz, *xp, *yp, *zp;
{
	*id1 = k;
	*id2 = seek_stept(stepz, k);
	magcar(Magd[j-1-cp], Incd[j-1-cp], Decd[j-1-cp], &xd, &yd, &zd);
	magcar(Magd[j], Incd[j], Decd[j], &xdd, &ydd, &zdd);
	*xp = xd - xdd;   *yp = yd - ydd;   *zp = zd - zdd;
}

/* seek_stept -- seek the target step of a pTRM test */

int seek_stept(stept, k)
int k;
double stept;
{
	int i = k;

	while (i > 0) {
		if (Step[i] == stept) return(i);
		i--;
	}
}

/* cal_mdtl -- calculate MD pTRM tail */

void cal_mdtl(j, k, m)
int j, k, m;
{
	Magn2[m] = Magd[j];   Incn2[m] = Incd[j];   Decn2[m] = Decd[j];
	vecsub(Magn2[m], Incn2[m], Decn2[m], Magn[k], Incn[k], Decn[k], Magmd+m, Incmd+m, Decmd+m);
	magcar(Magmd[m], Incmd[m], Decmd[m], Xmd+m, Ymd+m, Zmd+m);
}
#undef ERRDAT
