/* pinte -- analyse results of the Thellier experiment
		V.3.2.3 -- -C option is abandoned and reads Chi data automatically   051919
			   (compatibility with GMT 5.4.3, gcc 7.4.0, Ubuntu 18.04, ascertained)
		V.3.2.2 -- adapted to use with GMT 5.2.1 on Ubuntu 16.04   053118
		V.3.2.1 -- circle fit to data normalized by dNRM(I1--I2) & dTRM(I2--I1)   081614
			   (output of the results is only written in **.PI)
		V.3.2 -- circle fit by Taubin 1991 & M-command to norMalize TRM axis   072514
		V.3.1 -- circle fitting with params of Paterson11 & comd minor change   092613
		V.3.0.2 -- three NRM fractions were introduced   041713
		V.3.0.1 -- zigzag parameter Z* by Yu (2005) was introduced   102312
		V.3.0 -- New protocol for ZI(Coe), IZ(Aitken), & IZZI(Tauxe_Staudigel)   092106
		V.2.4.5 -- ERROR OF L WAS FIXED (linregy.c of stdsrc was error)   062806
		V.2.4.4 -- cdpt, devn, devl are introduced   072405
		V.2.4.3 -- my parameter dev is now normalized by L   072005
		V.2.4.2 -- parameters DRAT & CDRAT were added   061705
		V.2.4 -- command I to change over core and in situ data was added   081603
		V.2.3 -- optional PCA line on Zj-plot was added   040902
		V.2.2 -- optional chi vs step plot was introduced   030902
		V.2 -- merged from pintec & pintem   021201
	Usage, Options, Input & Output files:
		Omitted.  Please consult pi_help.h
	Basic idea of the program:
		pinte read only one file which includes data of multiple samples.
		name, azim, plun of (i+1)th sample are stored in Sampl[i], Azim[i], Plun[i].
		All other data are stored in series in Stepd[j], Magd[j], Incd[j], ...
		Which sample starts at which line is stored in Idsmpl[i].
		pi_ntp(i) take data of (i+1)th sample from Stepd[j], Magd[j], ...
		and stores them to Xn[k], Yn[k], ...  pi_gmtdat(i) sets up necessary GMT
		plot data, Ienn[k], Denn[k],... for the present (i+1)th sample.  
		Any plotting or calculation are made for the present sample only.  Hence,
		pinte keeps the processed data for only the present sample being analyzed.
	External variables for measurement data:
		Filed, Id_mthd[], Methd -- data file name, string & integer of method ID
		Nd -- total number of data or samples
		Nstep, Nptr, Nmdtl -- number of steps, pTRM test, & MD pTRM tail check for the present sample
		Sampl[i] -- name of (i+1)th sample
		Idsmpl[i] -- points beginning of (i+1)th sample on j of Magd[j], ...
		Azim[i], Plun[i] -- azimuth & plunge of (i+1)th sample
		Stepd[j] -- content of step in (j+1)th data line
		Magd[j], Incd[j], Decd[j] -- mag, inc, dec of (j+1)th data line
		Chid[j] -- chi of (j+1)th data line, set zero when not available
		Step[k] -- kth step (0th is NRM data)
		Xn[k], Yn[k], Zn[k] -- (x, y, z) of NRM vector for kth step
		Magn[k], Incn[k], Decn[k] -- (m, i, d) of NRM vector
		Xt[k], Yt[k], Zt[k] -- (x, y, z) of TRM vector for kth step
		Magt[k], Inct[k], Dect[k] -- (m, i, d) of TRM vector
		Xp[l], Yp[l], Zp[l] -- (x, y, z) of pTRM vector for (l+1)th pTRM test
		Magp[l], Incp[l], Decp[l] -- (m, i, d) of pTRM vector (l+1)th pTRM test
		Idpt1[l], Idpt2[l] -- FROM & TO steps of pTRM test in variable k.
		(Step[Idpt2[l]] indicates the step tested after heat to Step[Idpt1[l]])
		Magn2[m], Incn2[m], Decn2[m] -- (m+1)th 2nd NRM used for MD pTRM tail check
		Xmd[m], Ymd[m], Zmd[m] -- (x, y, z) of (m+1)th MD pTRM tail check
		Magmd[m], Incmd[m], Decmd[m] -- (m, i, d) of (m+1)th MD pTRM tail check
		Idmd[m] -- steps at which MD pTRM tail check was made in variable k.
		(Step[Idmd[m]] indicates the step tested for MD pTRM tail check of Xmd[m], ...)
		Chin[k],Chit[k],Chip[l],Chim[m] -- chi for various steps (not set when Chid[0]=0)
	External variables for GMT plot data:
		Fpt -- file pointer to a TMP file used in GMT commands
		Ia[k] -- array of k for kth step used for annotation
		Ienn[], Denn[], Ienr[] -- (inc,dec) & flipped inc for NRM equal area
		Iemn[], Demn[], Iemr[] -- ditto for MD pTRM tail check for NRM equal area
		Ietn[], Detn[], Ietr[] -- ditto for TRM equal area
		Iepn[], Depn[], Iepr[] -- ditto for pTRM tests
		Incs[], Decs[] -- temporary memory for in situ measurements data
		Xs[], Ys[], Zs[] -- ditto for any measurements data in (x,y,z)
		Xzn1, Xzn2, Yzn1, Yzn2 -- define the area plotted for NRM Zijderveld
		Yyr1, Yyr2, Yzr1, Yzr2 -- use for draw a PCA line on Zj plot
		Xzt1, Xzt2, Yzt1, Yzt2 -- ditto for TRM Zijderveld
		Gapzn, dXzn -- gap length & tick mark point for NRM Zijderveld
		Gapzt, dXzt -- ditto for TRM Zijderveld
		Xzn[], Yzn[], Zzn[] -- (x,y,z) on Zijderveld for NRM
		Xzm[], Yzm[], Zzm[] -- ditto for MD pTRM tail check
		Xzt[], Yzt[], Zzt[] -- ditto for TRM
		Xzp[], Yzp[], Zzp[] -- ditto for pTRM test data
		Ycn[], Yct[]  -- chi on chi-step curve for NRM & TRM steps
		Ycp[], Ycm[] -- ditto for pTRM & MD pTRM tail checks
		Ycmin, Ycmax -- lower & upper limits of chi-axis
		dYca, dYct -- annotation & tick intervals
		Xa[], Ya[] -- (TRM,NRM) for Arai daigram
		Xam[], Yam[] -- data point of MD pTRM tail check on Arai daigram
		Xap[], Yap[] -- data points of pTRM tests on Arai diagram
		Xac[], Xacm[], Xacp[] -- TRM-axis normalized by TRM_maximum
		NrmbarTmax -- NRM/TRM_Max when TRM-axis is normalized
		Scala, Scalax -- overall scale & NRM-axis sacle
		B, Sb, R -- slope, its sigma, correlation coefficient
		F, G, Q -- quality parameters by Coe et al. 1978
		Xl0, Yl0, Rl -- center & radius of fitted circle on Arai plot
		Kl, Sse -- k & SSE of fitted circle by Paterson 2011
*/

#include <ctype.h>
#include "pinte.h"

FILE *Fpt;
char **Sampl, **Stepd, Filed[WFILE], *Id_mthd[] = {"ZI", "IZ", "IZZI"}, \
	Buf[WBUF];
unsigned int Opt = 0, Methd = COE;
int Nd, Nstep, Nptr, Nmdtl, I1, I2, *Idsmpl, *Idpt1, *Idpt2, *Idmd, *Ia;
double *Azim, *Plun, *Magd, *Incd, *Decd, *Chid, *Step, *Xn, *Yn, *Zn, *Chin, \
	*Xmd, *Ymd, *Zmd, *Chim, *Xt, *Yt, *Zt, *Chit, *Xp, *Yp, *Zp, *Chip, \
	*Magn, *Incn, *Decn, *Magn2, *Incn2, *Decn2, *Magmd, *Incmd, *Decmd, \
	*Magt, *Inct, *Dect, *Magp, *Incp, *Decp;
double *Ienn, *Denn, *Ienr, *Iemn, *Demn, *Iemr, *Ietn, *Detn, *Ietr, \
	*Iepn, *Depn, *Iepr, Xzn1, Xzn2, Yzn1, Yzn2, Yyr1, Yyr2, Yzr1, Yzr2, \
	Gapzn, dXzn, *Xzn, *Yzn, *Zzn, *Xzm, *Yzm, *Zzm, Xzt1, Xzt2, Yzt1, Yzt2, \
	Gapzt, dXzt, *Xzt, *Yzt, *Zzt, *Xzp, *Yzp, *Zzp, *Ycn, *Yct, *Ycp, *Ycm, \
	Ycmax, Ycmin, dYca, dYct, *Xa, *Xac, *Xam, *Xacm, *Ya, *Yam, *Xap, *Xacp, *Yap, \
	Scala, Scalax, Scalaxc, NrmbarTmax, Factzjn, A, B, Sb, R, F, G, Q, L, \
	*Incs, *Decs, *Xs, *Ys, *Zs, Xl0, Yl0, Rl, Kl, Sse;

int main(argc, argv)
int argc;
char *argv[];
{
	FILE *fpr;
	int i = 0, c, nline;

	if ((fpr = pi_cmd(argc, argv, Filed)) == NULL) error(ERRFILE);
	nline = pi_dat(fpr);
	fclose(fpr);
	fprintf(stdout, "Thellier-%s experiment from %s\n", Id_mthd[Methd], Filed);
	Nstep = pi_ntp(i, &Nptr, &Nmdtl, &I1, &I2);
	fprintf(stdout, "%s\n", Sampl[i]);
	pi_gmtdat(i);
	Opt |= PLOT_CHI_BIT;
	pi_plot(i);
	sys_call(3, "evince ", FILEP, " &");
/*	sys_call(3, "gv -nodsc -watch -scale=0.7 ", FILEP, " &"); */
/*	sys_call(3, "gv -nodsc -watch ", FILEP, " &"); */
/*	sys_call(3, "gv --nodsc --watch ", FILEP, " &"); */
/*	sys_call(3, "ggv ", FILEP, " &"); */
	while ((c = pi_comd(COMMAND)) != ZERO){
		switch(toupper(c)){
		case '+':
			go_next(&i); break;
		case '-':
			go_back(&i); break;
		case '0':
			go_first(&i); break;
		case '9':
			go_last(&i); break;
		case 'L':
			do_regres(i); break;
		case 'C':
			do_circle(i); break;
		case 'S':
			change_scale(i); break;
		case 'N':
			normalize_trm(i); break;
		case 'I':
			change_orient(i); break;
		case 'A':
			change_anchor(i); break;
		case 'O':
			disable_annot(i); break;
		case 'T':
			change_ptlin(i); break;
		case 'Z':
			if (Opt & READ_CHI_BIT)
				zjt_or_chi(i);
			else
				fprintf(stdout, "No Chi data available for this sample!\n");
			break;
		case 'P':
			out_psfile(i); break;
		case 'H':
			fprintf(stdout, "%s\n", HELP2); break;
		default:
			break;
		}
	}
/*	free_all(Nd, nline); */
	quit_gv();
	sys_call(2, "rm -f ", FILET);
}

/* quit_gv -- force to kill the running gv (Now for evince) */

void quit_gv()
{
	FILE *fp;
	char *file1 = "PSINTE.TM2", *file2 = "PSINTE.TM3", s1[20];

	sys_call(2, "ps a >", file1);
	sys_call(4, "grep \"evince\" ", file1, " >", file2);
/*	sys_call(4, "grep \"gv -nodsc -watch -scale=0.7\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv -nodsc -watch\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv --nodsc --watch\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"ggv \" ", file1, " >", file2); */
	if ((fp = fopen(file2, "r")) == NULL) error("Error at quit_gv!");
	get_s(fp, 1, s1);
	fclose(fp);
	sys_call(2, "kill ", s1);
	sys_call(2, "rm -f ", file1);   sys_call(2, "rm -f ", file2);
}
