#include <string.h>
#include "pstat.h"

extern char **Group;
extern unsigned int Opt;
extern int Ngrp, Nsite, Nstat;
extern double Slnm, Mapln, Nplt, Npln;

static int c;

/* pst_comd -- returns a first letter of the hitted command */

int pst_comd(rem)
char *rem;
{
	fprintf(stdout, "%s ", rem);
	c = get_ch();
	if (c == ZERO) return(-1);
	else if (c == 'q' || c == 'Q') return ZERO;
	else return(c);
}

/* do_init -- called when the group is changed */

void do_init(n)
int n;
{
	fprintf(stdout, "%s\n", Group[n]);
	pst_grpdat(n);
	pst_plot(n);
}

/* go_next */

void go_next(n)
int *n;
{
	if (*n >= Ngrp -1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n += 1;
	do_init(*n);
}

/* go_back -- go back to the previous sample */

void go_back(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n -= 1;
	do_init(*n);
}

/* go_first */

void go_first(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n = 0;
	do_init(*n);
}

/* go_last */

void go_last(n)
int *n;
{
	if (*n >= Ngrp -1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n = Ngrp - 1;
	do_init(*n);
}

/* plot_locality -- draw mean or distinctive localities */

void plot_locality(n)
int n;
{
	if ( !(Opt & SHOW_SITE) ){
		Opt |= SHOW_SITE;
		Opt &= ~SHOW_DIST_SITE;
	}
	else if ( !(Opt & SHOW_DIST_SITE) )
		Opt |= SHOW_DIST_SITE;
	else {
		Opt &= ~SHOW_SITE;
		Opt &= ~SHOW_DIST_SITE;
	}
	pst_plot(n);
}

/* map_longitude -- set central longitude of equal area VGP map, or restore to that of site */

void map_longitude(n)
int n;
{
	if (Opt & MAP_LONG){
		Opt &= ~MAP_LONG;
		Mapln = Slnm;
		fprintf(stdout, "Longitude of map center was restored.\n");
	}
	else {
		while ((c=inpd("Longitude of center?",1,&Mapln)) != ZERO){
			if (c == -1) continue;
			else break;
		}
		if (c == ZERO) return;
		checklon(&Mapln, 0);
		Opt |= MAP_LONG;
	}
	pst_plot(n);
}

/* south_hemi -- change over the Northern & Southern Hemispheres to draw VGPs */

void south_hemi(n)
int n;
{
	if ( !(Opt & SOUTH_HEMI) )
		Opt |= SOUTH_HEMI;
	else
		Opt &= ~SOUTH_HEMI;
	pst_plot(n);
}

/* alph_on_off -- change over ON/OFF for alph95 together with ON/OFF for mean */

void alph_on_off(n)
int n;
{
	if ( !(Opt & SEPARATE_NS) ){
		if ( !(Opt & ALPH_FLD) ){
			Opt |= ALPH_FLD;
			Opt |= MEAN_VGP;
		}
		else {
			Opt &= ~ALPH_FLD;
			Opt &= ~MEAN_VGP;
		}
	}
	else {
		if ( !(Opt & ALPH_FLD) ){
			Opt |= ALPH_FLD;
			Opt &= ~MEAN_FLD;
		}
		else {
			Opt &= ~ALPH_FLD;
			Opt |= MEAN_FLD;
		}
	}
	pst_plot(n);
}

/* divide_ns -- change over combined or NS divided statistics */

void divide_ns(n)
int n;
{
	if ( !(Opt & SEPARATE_NS) ){
		Opt |= SEPARATE_NS;
		Opt &= ~ALPH_FLD;
		Opt |= MEAN_FLD;
		Opt |= MEAN_VGP;
	}
	else {
		Opt &= ~SEPARATE_NS;
		Opt |= ALPH_FLD;
		Opt &= ~MEAN_FLD;
		Opt |= MEAN_VGP;
	}
	pst_plot(n);
}

/* delete_one -- delete one farthest result from the Fisher statistics */

void delete_one(n)
int n;
{
	if (Nstat <= 2) return;
	pst_rejectf(n);
	pst_statf(n);
	pst_plot(n);
}

/* restore_one -- restore the nearest deleted data to the Fisher statistics */

void restor_one(n)
int n;
{
	if (Nstat >= Nsite) return;
	pst_restorf(n);
	pst_statf(n);
	pst_plot(n);
}

/* asd_north_pole(n) -- set ON/OFF to calculate ASD around north pole (Nplt,Npln)
	which can ba a user specified apparent pole
	possible aleration of the Fisher statistics is indicated   020725
*/

void asd_north_pole(n)
int n;
{
	if ( !(Opt & ASD_NPOLE) ){
		fprintf(stdout, "** ASD around a Fixed North Pole (%4.1f,%5.1f). **\n", Nplt, Npln);
		if (yorn("    Is this Fixed Pole OK? (Y or Rtn/N)", 1) == 0){
			while ((c=inpd("  * statistics might be altered with new NR limit *\n    lat & lon of the new fixed north pole?",2,&Nplt,&Npln)) != ZERO){
				if (c == -1) continue;
				else break;
			}
			if (c == ZERO) return;
		}
		Opt |= ASD_NPOLE;
	}
	else {
		fprintf(stdout, "ASD around the mean.\n");
		Opt &= ~ASD_NPOLE;
		Nplt = 90.;   Npln = 0.;
	}
	pst_statf(n);
	pst_plot(n);
}

/* axial_dpf -- plot fields in axial dipole field coordinate */

void axial_dpf(n)
int n;
{
	if ( !(Opt & AXIAL_DPF) )
		Opt |= AXIAL_DPF;
	else
		Opt &= ~AXIAL_DPF;
	pst_plot(n);
}

/* out_psfile -- copy the present plot to a specified file */

void out_psfile(n)
int n;
{
	char fileps[100];

	while ((c=inpstr("PS file?",fileps,WFILE)) != ZERO){
		if (strcmp(fileps,FILEP) == 0){
			fprintf(stdout, "Name other than the reserved file \"%s\"!\n", FILEP);
			continue;
		}
		if (fopen(fileps,"r") != NULL)
			if (yorn("File exists. Overwrite? (Y or Rtn/N)",+1) != 1)
				continue;
		sys_call(4, "cp -f ", FILEP, " ", fileps);
		break;
	}
}
