/* pst_dat -- read data from a file and stores to external variables Is[], Ds[], ...
	Internal variables:
		i -- counts number of groups
		j -- counts total number of sites
		nmax -- maximum of site numbers for a distinctive group
*/

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pstat.h"

extern char **Group, **Site;
extern unsigned int Opt;
extern int Ngrp, Nsite, Nstat, *Idsite, *Id, *Rjc;
extern double *Slt, *Sln, Sltm, Slnm, Mapln, Nplt, Npln, *Is, *Ds, *A95, *Nsmpl, *R, *K, \
		*Vlt, *Vln, *Vlnr, *Dlt, *Dltp, *Dltv, *Dlt2, *Dltp2, *Dltv2, \
		*Dq, *Ip, *Dp, *Dqp, *Sw, *Swv;

void pst_dat(fp, ntsite)
FILE *fp;
int *ntsite;
{

	char s1[50], s2[50], s3[10], s4[10], s5[10], s6[10], s7[10], s8[10], s9[10], s10[10], \
		s11[30], s12[10], s13[10], s14[10], s15[10];
	int nc, i = 0, j = 0, jj = 0, kk = 0, nmax = 0;

	while ((nc = get_s(fp,15,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13,s14,s15)) != EOF){
		if (nc == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp(s1, "$") == 0)
			i++;
		else
			j++;
	}
	Ngrp = i;   *ntsite = j;
	pst_mem1(Ngrp, *ntsite);
	i = 0;   j = 0;   rewind(fp);
	while ((nc = get_s(fp,15,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13,s14,s15)) != EOF){
		if (nc == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp(s1, "$") == 0){
			Idsite[i] = j;
			strcpy(Group[i], s2);
			if (jj > nmax) nmax = jj;   jj = 0;
			i++;
		}
		else {
			strcpy(Site[j], s1);
			Is[j] = atof(s2);   Ds[j] = atof(s3);
			A95[j] = atof(s4);   Nsmpl[j] = atof(s5);   R[j] = atof(s6);
			K[j] = atof(s7);   Vlt[j] = atof(s8);   Vln[j] = atof(s9);
			Sw[j] = atof(s12);   Swv[j] = atof(s13);
			Slt[j] = atof(s14);   Sln[j] = atof(s15);
			j++;   jj++;
		}
	}
	Idsite[i] = j;
	if (jj > nmax) nmax = jj;
	pst_mem2(nmax);
}

/* pst_grpdat -- set up & initialize the data of n th group for Fisher statistics */

void pst_grpdat(n)
int n;
{
	int i;
	double x = 0., y = 0., z = 0., dumx, dumy, dumz;

	Nstat = Nsite = Idsite[n+1] - Idsite[n];
	for (i = 0; i < Nsite; i++){
		Id[i] = Idsite[n] + i;   Rjc[i] = 0;
		fldadf(Slt[Id[i]],Sln[Id[i]],Nplt,Npln,Is[Id[i]],Ds[Id[i]],Ip+i,Dp+i);
		Dq[i] = 360. - Ds[Id[i]];   Dqp[i] = 360. - Dp[i];
		Vlnr[i] = Vln[Id[i]] + 180.;   checklon(Vlnr+i, 0);
		magcar(1., Slt[Id[i]], Sln[Id[i]], &dumx, &dumy, &dumz);
		x += dumx;   y += dumy;   z += dumz;
	}
	carmag(x, y, z, &dumx, &Sltm, &Slnm);
	if ( !(Opt & MAP_LONG) ) Mapln = Slnm;
	pst_statf(n);
}

/* pst_mem1 -- memory alloc 1 for data variables which are related to the raw dat
	ngrp, nsite are total no.s of gruoup & sites
*/

void pst_mem1(ngrp, nsite)
int ngrp, nsite;
{
	Group = mallocms(ngrp, WGRP);
	Idsite = mallocvi(ngrp+1);

	Site = mallocms(nsite, WSITE);
	Slt = mallocvd(nsite);   Sln = mallocvd(nsite);
	Is = mallocvd(nsite);   Ds = mallocvd(nsite);   A95 = mallocvd(nsite);
	Nsmpl = mallocvd(nsite);   R = mallocvd(nsite);   K = mallocvd(nsite);
	Vlt = mallocvd(nsite);   Vln = mallocvd(nsite);
	Sw = mallocvd(nsite);   Swv = mallocvd(nsite);
}

/* pst_mem2 -- memory alloc 2 for variables which are related to the present site
	nmax is maximum no of distinct sites for a group
*/

void pst_mem2(nmax)
int nmax;
{
	Id = mallocvi(nmax);   Rjc = mallocvi(nmax);
	Ip = mallocvd(nmax);   Dp = mallocvd(nmax);
	Dq = mallocvd(nmax);   Dqp = mallocvd(nmax);
	Vlnr = mallocvd(nmax);
	Dlt = mallocvd(nmax);   Dltp = mallocvd(nmax);   Dltv = mallocvd(nmax);
	Dlt2 = mallocvd(nmax);  Dltp2 = mallocvd(nmax);  Dltv2 = mallocvd(nmax);
}

/* free_all -- free memory for all external array variables -- DOES NOT WORK? */

void free_all(ngrp, nsite)
int ngrp, nsite;
{
	freems(ngrp, Group);
	freevi(Idsite);

	freems(nsite, Site);
	freevd(Slt);   freevd(Sln);
	freevd(Is);   freevd(Ds);   freevd(A95);
	freevd(Nsmpl);   freevd(R);   freevd(K);
	freevd(Vlt);   freevd(Vln);
	freevd(Sw);   freevd(Swv);

	freevi(Id);   freevi(Rjc);
	freevd(Ip);   freevd(Dp);
	freevd(Dq);   freevd(Dqp);
	freevd(Vlnr);
	freevd(Dlt);   freevd(Dltp);   freevd(Dltv);
	freevd(Dlt2);  freevd(Dltp2);  freevd(Dltv2);
}
