#include <string.h>
#include "psvar.h"

#define CONT	"Test of serial correlation by Watson & Beran (1967):\n\n\
Group: %s   N_site: %d   N_iteration: %d  File: %s\n\n\
L         = %.4f\n"

extern char **Group, Filed[WFILE];
extern unsigned int Opt;
extern int Nsite, *Id, Nwb;
extern double *Vlt, *Vln, L0, L950, L975;

/* psv_WB -- test of serial correlation by Watson & Beran 1967 */

int psv_WB(n)
int n;
{
	FILE *fp, *fp0, *fp1;
	char file[WFILE], file0[WFILE], file1[WFILE];
	static long idum[] = {-1};
	int *nc, *np, i, j, itmp, jdum1, jdum2;
	double *Lp, p950 = 0.95, p975 = 0.975;

	if (Nsite < 5){
		fprintf(stderr, "Nsite=%1d(<5)! Try next polarity group.\n", Nsite);
		Opt &= ~WATSON_BERAN;
		return ZERO;
	}
	Lp = mallocvd(Nwb+1);   nc = mallocvi(Nsite);   np = mallocvi(Nsite);
	strcpy(file, Group[n]);   strcat(file, ".WB");
	strcpy(file0, Group[n]);   strcat(file0, ".WB0");
	strcpy(file1, Group[n]);   strcat(file1, ".WB1");
	if ((fp = fopen(file, "w")) == NULL) error("error of fopen at psv_WatsonBeran!");
	if ((fp0 = fopen(file0, "w")) == NULL) error("error of fopen at psv_WatsonBeran!");
	if ((fp1 = fopen(file1, "w")) == NULL) error("error of fopen at psv_WatsonBeran!");
	Lp[0] = 0.;
	for (i=0; i<Nsite-1; i++)
		Lp[0] += vecumuls(Vlt[Id[i]], Vln[Id[i]], Vlt[Id[i+1]], Vln[Id[i+1]]);
	L0 = Lp[0];
	fprintf(stdout, "L         =%.4f (Nwb=%d)\n", L0, Nwb);
	fprintf(fp, CONT, Group[n], Nsite, Nwb, Filed, L0);
	fprintf(fp0, "%.4f\n", Lp[0]);   fprintf(fp1, "%.4f\n", Lp[0]);
	for (j=1; j<Nwb+1; j++){
		for (i=0; i<Nsite; i++) nc[i] =0;
		i = 0;
		while (i < Nsite){
			itmp = Nsite*ran1(idum);
			if (nc[itmp] == 1)
				continue;
			else {
				np[i] = itmp;
				nc[itmp] = 1;
				i++;
			}
		}
		Lp[j] = 0.;
		for (i=0; i<Nsite-1; i++)
			Lp[j] += vecumuls(Vlt[Id[np[i]]], Vln[Id[np[i]]], \
						Vlt[Id[np[i+1]]], Vln[Id[np[i+1]]]);
		fprintf(fp0, "%.4f\n", Lp[j]);
		fflush(fp0);
	}
	sort(Nwb, Lp+1);   jdum1 = Nwb*p950 + 1;   jdum2 = Nwb*p975 + 1;
	L0 = Lp[0];   L950 = Lp[jdum1];   L975 = Lp[jdum2];
	fprintf(stdout, "L(p=0.950)=%.4f\nL(p=0.975)=%.4f\n", L950, L975);
	fprintf(fp, "L(p=0.950)= %.4f\nL(p=0.975)= %.4f\n", L950, L975);
	for(j=1; j<Nwb+1; j++) fprintf(fp1, "%.4f\n", Lp[j]);
	fclose(fp);   fclose(fp0);   fclose(fp1);
	freevd(Lp);   freevi(nc);   freevi(np);
}
#undef CONT
/*
Next result LTN3 (Nsite=6) from lt_vgp.d was ascertained by hand:

	Lp[0]=  4.6308
	Lp[1]=  4.6939  2  0  4  3  5  1 
	Lp[2]=  4.7740  1  4  5  0  3  2 
	Lp[3]=  4.5102  1  2  5  4  0  3 
	Lp[4]=  4.3971  0  4  2  5  1  3 
	Lp[5]=  4.6947  2  0  3  1  5  4 
	Lp[6]=  4.6805  3  4  5  1  0  2 
	Lp[7]=  4.7091  2  4  3  0  1  5 
	Lp[8]=  4.5857  0  5  3  2  4  1 
	Lp[9]=  4.5453  5  3  1  0  2  4 
	Lp[10]= 4.5772  3  5  1  2  0  4 

	printf("Lp[%d]=%8.4f ", j, Lp[j]);
	for (i=0; i<Nsite; i++) printf("%2d ", np[i]);
	printf("\n");
*/
