#include <math.h>
#include "psvar.h"

extern unsigned int Opt;
extern int Nsite, Nstat, *Id, Nm;
extern double *Slt, *Sln, *Is, *Ds, *A95, *Nsmpl, *R, *K, *Vlt, *Vln, \
		*Sw, *Swv, *Dlt, *Dltv;
extern double Im, Dm, Vtm, Vnm, A95m, A95pm, A95vm, Rm, Rvm, \
		Km, Kvm, Nplt, Npln;
extern double Nwm, Swm, Swvm, Sb, Sbv, St, Stv;

static int k;
static double x, y, z, xv, yv, zv;

/* psv_statf -- Fisher statisitcs of site mean directions & VGPs */

void psv_statf(n)
int n;
{
	int i;
	double d;

	if (Nsite <= 1){
		set_one_site();   return;
	}
	summ_data();
	if ((Nstat = k) == 1) error("psv_statf: error of Nstat=1!");
	statf_result(k, x, y, z, &Im, &Dm, &Nm, &A95m, &Rm, &Km);
	statf_result(k, xv, yv, zv, &Vtm, &Vnm, &d, &A95vm, &Rvm, &Kvm);
	Nwm /= Nm;   Swm /= Nm;   Swvm /= Nm;
	cal_delta();
	cal_asd();
	St /= (Nm-1);   Sb = ((d=St-Swm/Nwm) > 0.) ? sqrt(d) : -1.;   St = sqrt(St);
	Stv /= (Nm-1);   Sbv = ((d=Stv-Swvm/Nwm) > 0.) ? sqrt(d) : -1.;   Stv = sqrt(Stv);
	Swm = sqrt(Swm);   Swvm = sqrt(Swvm);
}

/* set_one_site */

void set_one_site()
{
	Im = Is[Id[0]];   Dm = Ds[Id[0]];
	Nm = 1;   A95m = -1.;   Rm = 1.;   Km = -1.;
	Vtm = Vlt[Id[0]];   Vnm = Vln[Id[0]];
	A95vm = -1.;   Rvm = 1.;   Kvm = -1.;
	Dlt[0] = Dltv[0] = 0.;
	Swm = Sw[Id[0]];   Swvm = Swv[Id[0]];
	St = Stv = -1.;
	Sb = Sbv = -1.;
}

/* summ_data */

void summ_data()
{
	int i;
	double a, b, c, av, bv, cv;

	k = 0;
	x = y = z = 0.;
	xv = yv = zv = 0.;
	Nwm = 0.;
	Swm = Swvm = 0.;
	for (i = 0; i < Nsite; i++){
		magcar(1., Is[Id[i]], Ds[Id[i]], &a, &b, &c);
		magcar(1., Vlt[Id[i]], Vln[Id[i]], &av, &bv, &cv);
		Nwm += Nsmpl[Id[i]];
		Swm += Sw[Id[i]]*Sw[Id[i]];   Swvm += Swv[Id[i]]*Swv[Id[i]];
		x += a;   y += b;   z += c;
		xv += av;   yv += bv;   zv += cv;
		k++;
 	}
}

/* statf_result */

void statf_result(k, x, y, z, inc, dec, n, a95, r, km)
int k, *n;
double x, y, z, *inc, *dec, *a95, *r, *km;
{
	double m, cosa, cosi;

	m = *n = k;
	if (*n == 0){
		*inc = 0.;   *dec = 0.;
		*a95 = -1.;   *r = -1.;   *km = -1.;
		return;
	}
	carmag(x, y, z, r, inc, dec);
	if (*n == 1){
		*a95 = -1.;   *km = -1.;
		return;
	}
	*km = (m - 1.)/(m - (*r));
	cosa = 1.  -  (m - (*r))/(*r)*(pow(20., 1./(m-1.)) - 1.);
	if (cosa > 1.) *a95 = 0.;
	else if (cosa < -1.) *a95 = 180.;
	else *a95 = acos(cosa)*RAD;
}

/* cal_delta */

void cal_delta()
{
	int i;
	double dumi, dumd;

	for (i = 0; i < Nsite; i++){
		if ( !(Opt & ASD_NPOLE) ){
			Dlt[i] = angdif(1., Is[Id[i]], Ds[Id[i]], 1., Im, Dm);
			Dltv[i] = angdif(1., Vlt[Id[i]], Vln[Id[i]], 1., Vtm, Vnm);
		}
		else {
			vgpadf(Slt[Id[i]], Sln[Id[i]], Nplt, Npln, &dumi, &dumd);
			Dlt[i] = angdif(1., Is[Id[i]], Ds[Id[i]], 1., dumi, dumd);
			Dltv[i] = angdif(1., Vlt[Id[i]], Vln[Id[i]], 1., Nplt, Npln);
		}
	}
}

/* cal_asd */

void cal_asd()
{
	int i, j = 0;

	St = Stv = 0.;
	for (i = 0; i < Nsite; i++){
		St += Dlt[j]*Dlt[j];
		Stv += Dltv[j]*Dltv[j];
		j++;
	}
}
