/* psvar -- plot vgps from lava succession
	V.1.1 -- modified to use with GMT 5.2.1 and evince   053018
	V.1.0 -- pstat was altered to analyze Icelandic lava succession data   122713

	Memo on the basic idea (example of 37 lavas with 3 polarities):

	 j ID[]  Idsite[]   polarity group
	------------------------------------
	 0=ID[0]=Idsite[0]  Normal lavas 1
	  ...               Nsite=13
	12=ID[12]
	------------------------------------
	13=ID[0]=Idsite[1]  Reversed lavas 1
	  ...               Nsite=15
	27=ID[14]
	------------------------------------
	28=ID[0]=Idsite[2]  Normal lavas 2
	  ...               Nsite=9
	36=Id[8]
	------------------------------------
	37   =   Idsite[3]  Ngrp=3

	For some other explanation, see the HELP message contained in psv_cmd()
*/

#define COMMAND "+,-,0,9,C,G,U,A,M,V,D,I,S,L,W,N,P,Z,H,Q?"
#define HELPCMD "Commands:\n\
  \e[1m+\e[m-next, \e[1m-\e[m-back, \e[1m0\e[m-first, \e[1m9\e[m-last, lo\e[1mC\e[mality-on/on2/off, map-lon\e[1mG\e[mitude,\n\
  map-base-\e[1mL\e[matitude, so\e[1mU\e[mthern-hemi-on/off, \e[1mA\e[mlph95-on/off, \e[1mM\e[mean-on/off,\n\
  \e[1mV\e[mgp-path-on/off, \e[1mD\e[mel-pre/post-vgps, site-\e[1mI\e[md-num-on/off, a\e[1mS\e[md-pole,\n\
  \e[1mW\e[matson-beran-on/off, \e[1mN\e[mum-iteration, \e[1mP\e[msfile, \e[1mZ\e[m-gmt-stamp, \e[1mH\e[melp, \e[1mQ\e[muit"

#include <ctype.h>
#include "psvar.h"

FILE *Fpt;
char **Group, **Site, Filed[WFILE], Buf[WBUF];
unsigned int Opt = 0;
int Ngrp, Nsite, Nstat, *Numbr, *Idsite, *Id, Nm, Nwb = 2500;
double *Slt, *Sln, Sltm, Slnm, Mapln, Mapltc, Nplt, Npln, *Is, *Ds, *Ig, *Dg, \
	*A95, *Nsmpl, *R, *K, *A95v, *Vlt, *Vln, *Sw, *Swv, *Dlt, *Dltv;
double Im, Dm, Vtm, Vnm, A95m, A95vm, Rm, Rvm, Km, Kvm, Nwm, Swm, Swvm, \
	Sb, Sbv, St, Stv, L0, L950, L975;

int main(argc, argv)
int argc;
char *argv[];
{
	FILE *fpr, *psv_cmd();
	int n = 0, c, ntsite;

	Opt |= SHOW_SITE;  Opt |= PLOT_ALPH;  Opt |= PLOT_PATH;  Opt |= SHOW_PRE_POST;
	Mapltc = 0.;   Nplt = 90.;   Npln = 0.;
	if ((fpr = psv_cmd(argc, argv, Filed)) == NULL) error("Error in fopen!");
	psv_dat(fpr, &ntsite);
	fclose(fpr);
	fprintf(stdout, "Paleosecular variation from %s\n", Filed);
	do_init(n);
	sys_call(3, "evince ", FILEP, " &");
/*	sys_call(3, "gv -nodsc -watch -scale=0.7 ", FILEP, " &"); */
/*	sys_call(3, "gv -nodsc -watch ", FILEP, " &"); */
/*	sys_call(3, "gv --nodsc --watch ", FILEP, " &"); */
	while ((c = psv_comd(COMMAND)) != ZERO){
		switch(toupper(c)){
		case '+':
			go_next(&n); break;
		case '-':
			go_back(&n); break;
		case '0':
			go_first(&n); break;
		case '9':
			go_last(&n); break;
		case 'C':
			plot_locality(n); break;
		case 'G':
			map_longitude(n); break;
		case 'L':
			map_cutoff_lat(n); break;
		case 'U':
			change_south_hemi(n); break;
		case 'A':
			alph_on_off(n); break;
		case 'M':
			plot_mean(n); break;
		case 'V':
			plot_path(n); break;
		case 'D':
			delete_pre_post(n); break;
		case 'I':
			plot_id(n); break;
		case 'S':
			asd_north_pole(n); break;
		case 'W':
			do_WB(n); break;
		case 'N':
			num_iteration(n); break;
		case 'P':
			out_psfile(n); break;
		case 'Z':
			plot_stamp(n); break;
		case 'H':
			fprintf(stdout, "%s\n", HELPCMD); break;
		default:
			break;
		}
	}
/*	free_all(Ngrp, ntsite); */
	quit_gv();
	sys_call(2, "rm -f ", FILET);
}

/* quit_gv -- force to kill the running Ghost View */

void quit_gv()
{
	FILE *fp;
	char *file1 = "PSVAR.TM2", *file2 = "PSVAR.TM3", s1[20];

	sys_call(2, "ps a >", file1);
	sys_call(4, "grep \"evince\" ", file1, " >", file2);
/*	sys_call(4, "grep \"gv -nodsc -watch -scale=0.7\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv -nodsc -watch\" ", file1, " >", file2); */
/*	sys_call(4, "grep \"gv --nodsc --watch\" ", file1, " >", file2); */
	if ((fp = fopen(file2, "r")) == NULL) error("Error at quit_gv!");
	get_s(fp, 1, s1);
	fclose(fp);
	sys_call(2, "kill ", s1);
	sys_call(2, "rm -f ", file1);   sys_call(2, "rm -f ", file2);
}
#undef COMMAND
#undef HELPCMD
