#include <math.h>
#include "std.h"

#define EPS 1.e-8

static int f1, f2;
static double pp;

/* ftab -- critical F at which the probability for F being such large is p   112798
	find a root of the next equation by calling rtsafe()
		f(F) = p - fdist(n1,n2,F) = 0
*/

double ftab(n1, n2, p)
int n1, n2;
double p;
{
	void f_F();
	double rl = 0.1, rh = 1.e6;   /* should cover the following n1,n2,p */

	if (n1 < 1 || n2 < 1) error("ftab: less than 1 degrees of freedom!");
	if (n1 > 1000 || n2 > 1000) error("ftab: too large degrees of freedom (>1000)!");
	if (p > 1.) error("ftab: p is larger than 1!");
/*	if (p < 0.001) error("ftab: too small p (<0.001)!"); */
	if (p < 0.0005) error("ftab: too small p (<0.0005)!");
	if (p <= 0.5){
		f1 = n1;   f2 = n2;   pp = p;
		return rtsafe(f_F, rl, rh, EPS);
	}
	else {
		f1 = n2,   f2 = n1;   pp = 1. - p;
		return 1./rtsafe(f_F, rl, rh, EPS);
	}
}


/* fdist -- probability for F to exceed the critical value
	n1,n2: degrees of freedom    F: critical value of F (>=0)
*/

double fdist(n1, n2, F)
int n1, n2;
double F;
{
	double a, b;

	a = n1;   b = n2;
	if (n1 < 1 || n2 < 1) error("fdist: less than 1 degrees of freedom!");
	if (F < 0.) error("fdist: negative F!");
	return betai(b/2., a/2., b/(b+a*F));
}


/* f_F -- function to determine the root for ftab */

void f_F(x, f, df)
double x, *f, *df;
{
	double a, b, ah, bh, c, d, e, fdist();

	*f = pp - fdist(f1, f2, x);
	a = f1;   b = f2;
	ah = a/2.;   bh = b/2.;
	c = exp(gammln(ah+bh) - gammln(ah) - gammln(bh));
	d = a*x/(b + a*x);   e = b/(b + a*x);
	*df = c*pow(d,ah)*pow(e,bh)/x;
}
#undef EPS
/*
printf("f_F:x,f,df,c,d,e; %f %e %e %e %e %e\n", x, *f, *df, c, pow(d,ah), pow(e,bh));
*/
