#include <math.h>
#include "std.h"

#define ROTATE(a,i,j,k,l) g=*(*a+i*(n+1)+j);h=*(*a+k*(n+1)+l);\
	*(*a+i*(n+1)+j)=g-s*(h+g*tau);*(*a+k*(n+1)+l)=h+s*(g-h*tau);

/* jacobi -- Jacobi's method for eigenvalues and eigenvectors   0597,0896
	minor change in memory allocation   0819
	modified from Press et al. 1992, Numerical Recepies in C, p.467
	For simplicity, you should allocate vectors and matrixes as
		(n+1) and (n+1)x(n+1)
	and use them only (1) to (n) and (1,1) to (n,n)
    *** In the host, DO NOT USE "mallocmd" to alloc mem "a" & "v" (see, statb) ***
	returns ZERO when too many iteration, otherwise returns +1.
*/

int jacobi(a, n, d, v, nrot)
int n, *nrot;	/* n is a math dimension not a memory dimension */
double **a, *d, **v;
{
	int j, iq, ip, i;
	double tresh, theta, tau, t, sm, s, h, g, c, *b, *z;

	b = mallocvd(n+1);   z = mallocvd(n+1);
	for (ip = 1; ip <= n; ip++) {
		for (iq = 1; iq <= n; iq++) *(*v + ip*(n+1) + iq) = 0.;
		*(*v + ip*(n+1) + ip) = 1.;
	}
	for (ip = 1; ip <= n; ip++) {
		*(b + ip) = *(d + ip) = *(*a + ip*(n+1) + ip);
		*(z + ip) = 0.;
	}
	*nrot = 0;
	for (i = 1; i <= 50; i++) {
		sm = 0.;
		for (ip = 1; ip <= n-1; ip++) {
			for (iq = ip+1; iq <= n; iq++)
				sm += fabs(*(*a + ip*(n+1) + iq));
		}
		if (sm == 0.0) {
			freevd(z);   freevd(b);
			return(1);
		}
		if (i < 4)
			tresh = 0.2*sm/(n*n);
		else
			tresh = 0.0;
		for (ip = 1; ip <= n-1; ip++){
			for (iq = ip+1; iq <= n; iq++){
				g = 100.*fabs(*(*a + ip*(n+1) + iq));
				if ( i > 4 && fabs(*(d+ip))+g == fabs(*(d+ip))
					&& fabs(*(d+iq))+g == fabs(*(d+iq)) )
					*(*a + ip*(n+1) + iq) = 0.;
				else if ( fabs(*(*a + ip*(n+1) + iq)) > tresh ){
					h = *(d+iq) - *(d+ip);
					if ( fabs(h)+g == fabs(h) )
						t = (*(*a + ip*(n+1) + iq))/h;
					else {
						theta = 0.5*h/(*(*a + ip*(n+1) + iq));
						t = 1./(fabs(theta) + sqrt(1. + theta*theta));
						if (theta < 0.) t = -t;
					}
					c = 1./sqrt(1. + t*t);
					s = t*c;
					tau = s/(1. + c);
					h = t*(*(*a + ip*(n+1) + iq));
					*(z+ip) -= h;
					*(z+iq) += h;
					*(d+ip) -= h;
					*(d+iq) += h;
					*(*a + ip*(n+1) + iq) = 0.;
					for (j = 1; j <= ip-1; j++){
						ROTATE(a, j, ip, j, iq)
					}
					for (j = ip+1; j <= iq-1; j++){
						ROTATE(a, ip, j, j, iq)
					}
					for (j = iq+1; j <= n; j++){
						ROTATE(a, ip, j, iq, j)
					}
					for (j = 1; j <= n; j++){
						ROTATE(v, j, ip, j, iq)
					}
					++(*nrot);
				}
			}
		}
		for (ip = 1; ip <= n; ip++) {
			*(b+ip) += *(z+ip);
			*(d+ip) = *(b+ip);
			*(z+ip) = 0.;
		}
	}
	fprintf(stderr, "jacobi: too many iterations!\n");
	freevd(z);   freevd(b);
	return(ZERO);	/* too many iterations */
/*	error("Too many iterations in routine jacobi"); */
}

/* eigenorder -- order eigen values. */

void eigenorder(d, i1, i2, i3)
int *i1, *i2, *i3;
double *d;
{
	*i1 = 1;
	if (d[*i1] < d[2])
		*i2 = 2;
	else {
		*i2 = *i1;
		*i1 = 2;
	}
	if (d[*i2] < d[3])
		*i3 = 3;
	else if (d[3] < d[*i1]){
		*i3 = *i2;
		*i2 = *i1;
		*i1 = 3;
	}
	else {
		*i3 = *i2;
		*i2 = 3;
	}
}
#undef ROTATE
