/* quad2ds -- 2 dimensional integral by independent variables x and y   8/17/96
		based on Simpson's rule (should be MORE ACCURATE than quad2dg)
	func: function of x and y
	integral by x: from x1 to x2  ** these intervals are independent, so  **
	integral by y: from y1 to y2  ** do not integrate on complicated area **
	minor amendments in the defined constatnts  0819
*/

#include <math.h>
#include "std.h"

#define EPS 1.0e-6
#define JMAX 20

static double (*nrfunc)(), x1sav, x2sav, ysav;

double quad2ds(func, x1, x2, y1, y2)
double (*func)(), x1, x2, y1, y2;
{
	double quad2ds1();

	x1sav = x1;
	x2sav = x2;
	nrfunc = func;
	return qsimp(quad2ds1, y1, y2);	/* y-integral by qsimp() is the last */
}

double quad2ds1(y)
double y;
{
	double quad2ds3(), quad2ds2();

	ysav = y;
	return quad2ds3(quad2ds2, x1sav, x2sav); /* x-integral is the first */
}

double quad2ds2(x)
double x;
{
	return (*nrfunc)(x, ysav);
}

/* quad2ds3 -- equivalent to qsimp() (should be INDEPENDENT from qsimp) */

double quad2ds3(func, a, b)
double (*func)(), a, b;
{
	int j;
	double s, st, ost, os, quad2ds4();

	ost = os = -1.0e30;
	for (j = 1; j <= JMAX; j++){
		st = quad2ds4(func, a, b, j);
		s = (4.*st - ost)/3.;
		if ( fabs(s - os) < EPS*fabs(os) ) return s;
		os = s;
		ost = st;
	}
	return 0.;
}

/* quad2ds4 -- equivalent to trapzd() (for the same reason as above) */

double quad2ds4(func, a, b, n)
int n;
double (*func)(), a, b;
{
	double x, tnm, sum, del;
	static double s;
	int it, j;

	if (n == 1)
		return ( s = 0.5*(b - a)*( (*func)(a) + (*func)(b) ) );
	else {
		for (it = 1, j = 1; j < n-1; j++) it <<= 1;
		tnm = it;
		del = (b - a)/tnm;
		x = a + 0.5*del;
		for (sum = 0.0, j = 1; j <= it; j++, x += del)
			sum += (*func)(x);
		s = 0.5*(s + (b - a)*sum/tnm);
		return s;
	}
}
#undef EPS
#undef JMAX
