/* sort, sort2 -- Quicksort in ascending order by Numerical Recipes, with small adaption  0900
	Usage:
		use normally with n elements of array (a[0], a[1], ..., a[n-1])
	How it works:
		using a pointer *aa which points one element below, call Recipes's
		original fuction as sort_r(n, aa) with n elements of displaced
		array (aa[1], aa[2], ...., aa[n])
	Changes in original function sort_r() & sort2_r():
		float variables were changed to double
		memory allocation of istack[] was made by directly calling malloc()
*/

#include "std.h"

/* #define NRANSI */
#define SWAP(a,b) temp=(a);(a)=(b);(b)=temp;
#define M 7
#define NSTACK 50

static double *aa, *bb;


/* sort -- sort array a[] */

void sort(n, a)
int n;
double *a;
{
	aa = a - 1;
	sort_r(n, aa);
}

/* sort2 -- sort array a[] & b[], according to a[] */

void sort2(n, a, b)
int n;
double *a, *b;
{
	aa = a - 1;   bb = b - 1;
	sort2_r(n, aa, bb);
}

/* sort_r */

void sort_r(unsigned long n, double arr[])
{
	unsigned long i,ir=n,j,k,l=1;
	int jstack=0,*istack;
	double a,temp;

/*
	if ((istack = (int *) malloc((size_t)((NSTACK+1)*sizeof(int)))) == NULL)
		error("Error in malloc of istack!");
*/
	istack = mallocvi(NSTACK+1);
	for (;;) {
		if (ir-l < M) {
			for (j=l+1;j<=ir;j++) {
				a=arr[j];
				for (i=j-1;i>=1;i--) {
					if (arr[i] <= a) break;
					arr[i+1]=arr[i];
				}
				arr[i+1]=a;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		} else {
			k=(l+ir) >> 1;
			SWAP(arr[k],arr[l+1])
			if (arr[l+1] > arr[ir]) {
				SWAP(arr[l+1],arr[ir])
			}
			if (arr[l] > arr[ir]) {
				SWAP(arr[l],arr[ir])
			}
			if (arr[l+1] > arr[l]) {
				SWAP(arr[l+1],arr[l])
			}
			i=l+1;
			j=ir;
			a=arr[l];
			for (;;) {
				do i++; while (arr[i] < a);
				do j--; while (arr[j] > a);
				if (j < i) break;
				SWAP(arr[i],arr[j]);
			}
			arr[l]=arr[j];
			arr[j]=a;
			jstack += 2;
			if (jstack > NSTACK) error("NSTACK too small in sort_r.");
			if (ir-i+1 >= j-l) {
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} else {
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	freevi(istack);
}

/* sort2_r */

void sort2_r(unsigned long n, double arr[], double brr[])
{
	unsigned long i,ir=n,j,k,l=1;
	int *istack,jstack=0;
	double a,b,temp;

/*
	if ((istack = (int *) malloc((size_t)((NSTACK+1)*sizeof(int)))) == NULL)
		error("Error in malloc of istack!");
*/
	istack = mallocvi(NSTACK+1);
	for (;;) {
		if (ir-l < M) {
			for (j=l+1;j<=ir;j++) {
				a=arr[j];
				b=brr[j];
				for (i=j-1;i>=1;i--) {
					if (arr[i] <= a) break;
					arr[i+1]=arr[i];
					brr[i+1]=brr[i];
				}
				arr[i+1]=a;
				brr[i+1]=b;
			}
			if (!jstack) {
				freevi(istack);
				return;
			}
			ir=istack[jstack];
			l=istack[jstack-1];
			jstack -= 2;
		} else {
			k=(l+ir) >> 1;
			SWAP(arr[k],arr[l+1])
			SWAP(brr[k],brr[l+1])
			if (arr[l+1] > arr[ir]) {
				SWAP(arr[l+1],arr[ir])
				SWAP(brr[l+1],brr[ir])
			}
			if (arr[l] > arr[ir]) {
				SWAP(arr[l],arr[ir])
				SWAP(brr[l],brr[ir])
			}
			if (arr[l+1] > arr[l]) {
				SWAP(arr[l+1],arr[l])
				SWAP(brr[l+1],brr[l])
			}
			i=l+1;
			j=ir;
			a=arr[l];
			b=brr[l];
			for (;;) {
				do i++; while (arr[i] < a);
				do j--; while (arr[j] > a);
				if (j < i) break;
				SWAP(arr[i],arr[j])
				SWAP(brr[i],brr[j])
			}
			arr[l]=arr[j];
			arr[j]=a;
			brr[l]=brr[j];
			brr[j]=b;
			jstack += 2;
			if (jstack > NSTACK) error("NSTACK too small in sort2.");
			if (ir-i+1 >= j-l) {
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} else {
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
}
#undef M
#undef NSTACK
#undef SWAP
/* #undef NRANSI */
/* (C) Copr. 1986-92 Numerical Recipes Software SD&5-X=!+!.
istack=ivector(1,NSTACK);
free_ivector(istack,1,NSTACK);
*/
