#include <math.h>
#include "std.h"

#define EPS 1.e-8

static double g, pp;

/* ttab -- critical t at which the probability for t being such large is p   081403
	(upper percentage point of the student's t distribution)
	find a root of the next equation by calling rtsafe()
		f(t) = p - tdist(n,t) = 0
	CAUTION: the degrees of freedom is changed from int n to double nd!   092318
*/

double ttab(nd, p)
double nd, p;
{
	void f_t();
	double rl = -1.e-20, rh = 1.e6;   /* should cover the following nd, p */

	if (nd < 1.) error("ttab: less than 1 degrees of freedom!");
	if (p > 1.) error("ttab: p is larger than 1!");
	if (p < 0.00001) error("ttab: too small p (<0.00001)!");
	if (p <= 0.5){
		g = nd;   pp = p;
		return rtsafe(f_t, rl, rh, EPS);
	}
	else {
		g = nd,   pp = 1. - p;
		return -rtsafe(f_t, rl, rh, EPS);
	}
}

/* tdist -- probability for t to exceed the critical value.  
	p(t,g) used in Table 3 of Statistics Manual by Crow et al. (1960) should relate to
	A(t,g) of (6.4.7) of Numerical Recipes in C by Press et al. (1992) in the following way.
		p(t,g) = (1 - A(t,g))/2
	where g is the degrees of freedom
	CAUTION: the degrees of freedom is changed from int n to double nd!   092318
*/

double tdist(nd, t)
double nd, t;
{
	if (nd < 1.) error("tdist: less than 1 degrees of freedom!");
	return 0.5*betai(nd*0.5, 0.5, nd/(nd + t*t));
}

/* f_t -- function to determine the root for ttab */

void f_t(x, f, df)
double x, *f, *df;
{
	*f = pp - tdist(g, x);
	*df = 0.5*betai(g*0.5, 0.5, g/(g + x*x));
}
#undef EPS
